<?php

namespace Teknisa\OPL\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\OPL\Factories\Dfs48000SaidaDoOperadorLogisticoFactory;
use Teknisa\OPL\Factories\Dfs48000EntradaDoOperadorLogisticoFactory;

use Teknisa\OPL\Services\LancamentoNotafiscalService;
use Teknisa\PebLibs\Services\ValidacoesEstoqueService;

use Teknisa\Libs\Util\Masks\PictureProduto; 
use Teknisa\Libs\Util\Masks\Masks;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Environment;
use Estoque\Entities\Lanctoestoq;
use Estoque\Entities\Movilote;
use Geral\Entities\Produto;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
    
class Dfs48000SaidaDoOperadorLogisticoService {
    
    const PROCEDURE_GERA_PRODUTO            = 'GERA_NOTAFISCAL_PRODUTO';
    const PROCEDURE_CONSULTA_GRUPOFISCAL    = 'CONSULTA_GRUPOFISCAL';
    const PROCEDURE_ATUALIZA_ESTOQUE        = 'ATUALIZA_ESTOQUE';
    const NOVO_CODIGO_MOVOCORR              = 'MOVOCORR';
    const NOVO_CODIGO_LANC_ESTOQUE          = 'LANCTOEST';
    const NOVO_CODIGO_NOTA_LANC             = 'LANCNF/FILIAL';
    const NOVO_CODIGO_NRPEXOL               = 'PEDIDOEXOL';
    const PROCEDURE_GERA_NOTA_CORPO_ENTRADA = 'GERA_NOTAFISCAL_CORPO_ENTRADA';
    const PROCEDURE_GERA_NOTA_CORPO         = 'GERA_NOTAFISCAL_CORPO';
    const PROCEDURE_IMPOSTO_ENTRADA_PRODUTO = 'IMPOSTO_ENTRADA_PRODUTO';
    const PROCEDURE_GERA_FECHAMENTO_ENTRADA = 'GERA_FECHAMENTO_NFENTRADA';
    const PROCEDURE_CONSULTA_INCIDENCIA_IMP = 'CONSULTA_INCIDENCIA';
    const PROCEDURE_CONSULTA_ALIQUOTA       = 'CONSULTA_ALIQUOTA';
    const PROCEDURE_BAIXA_AF                = 'NF_BAIXA_AF';
    
    const MSG_ERROS = 
        array("DAT_MAI"          => "Data informada não deve ser maior do que a data de processamento",
              "DAT_MEN"          => "Data informada não deve ser menor do que a data de processamento",
              "DAT_M"            => "Data informada fora do período permitido.",
              "SERIE_PAD"        => "Não existe série padrão para filial.",
              "LANC_FORTADT"     => "Data de saída fora do periodo de integração.",
              "LANC_N_GER"       => "Não existem itens cuja quantidade devolvida é maior do que 0.",
              "TOTAL_NF_ZERADO"  => "Existem itens cujo total da NF de saída está zerado.",
              "DTEMISSAO_VAZIA"  => "Nota Fiscal ainda não foi aprovada(sem data de emissão).",
              "NRNOTAFISC_VAZIO" => "Nota Fiscal não pode ter data de saída menor do que a data de emissão de saída.", 
              "QTDE_ZERADA"      => "Quantidade do item devolvida não foi informada ou está zerada.",
              "PRECO_ZERADO"     => "Existem itens cujo preço ficaria zerado.",
              "PRODUTO_FILIAL"   => "Produto não existe para a filial.",
              "PRODUTO_INATIVO"  => "Produto inativo no sistema.",
              "PRODUTO_INATFIL"  => "Produto inativo para filial.",
              "EST_INSUFICIENTE" => "Estoque do item insuficiente para devolução.",
              "ERRO_TOT"         => "Valor total da NF de saÍda diferente do valor total da NF de saída.",
              "SEMDOCUFISC"      => "Modelo de documento fiscal não cadastrado.",
              "ITEMSEMCFOP"      => "CFOP não parametrizado para o item.");
              
    protected $dfs48000SaidaDoOperadorLogisticoService;
    protected $NewCode;
    protected $lancamentoNotaFiscalService;
    
    private $arrayNotasErro             = array();
    private $arrayNotasErroEntrada      = array();
    private $arrayProdutosErro          = array();
    private $arrayProdutosErroEntrada   = array();
    private $arrayNotasSucesso          = array();
    private $arrayNFEErro               = array();
    private $arrayNFESucesso            = array();
    private $notasSucesso               = 0;
    private $prodImpos;
    
    public function __construct(Dfs48000SaidaDoOperadorLogisticoFactory $dfs48000SaidaDoOperadorLogisticoFactory, EntityManager $entityManager, NewCode $NewCode, LancamentoNotafiscalService $lancamentoNotaFiscalService, Environment $environment, ValidacoesEstoqueService $validacoesEstoqueService, Dfs48000EntradaDoOperadorLogisticoFactory $Dfs48000EntradaDoOperadorLogisticoFactory) {
        $this->dfs48000SaidaDoOperadorLogisticoFactory = $dfs48000SaidaDoOperadorLogisticoFactory;
        $this->Dfs48000EntradaDoOperadorLogisticoFactory = $Dfs48000EntradaDoOperadorLogisticoFactory;
        $this->connection   = $entityManager->getConnection();
        $this->NewCode   = $NewCode;
        $this->lancamentoNotaFiscalService = $lancamentoNotaFiscalService;
        $this->validacoesEstoqueService    = $validacoesEstoqueService;
        $this->environment  = $environment;
        $this->masks = new Masks();
    }
    
    
    public function geraCorpoNotaEntrada($cdfilial, $cdfilirece, $cdempresa, $cdtipooper, $cdserieentr, $cdcliente, $sistema, $atualizaestoque, $nrlanctonf, $pesstrans, $insctrans, $placaveiculo, $ufveiculo, $pgtofrete, $quantidadedevolu, $espefvolu, $marcvolu, $numevolu, $nrlancestoq, $dtentrsaid, $vrbrutovolu, $vrliquidovolu, $vrnotafisc, $nrlancorig, $dtlancmennf, $dtemissao, $nrnotafisc, $nracessonfe='', $cdmododocfisc='') {
        $dtemissao  = str_replace("-", "/", $dtemissao);
        $dtentrsaid = str_replace("-", "/", $dtentrsaid);
        $prxLanctoNf    = NULL;
        $prxLanctoEstoq = NULL;
        
        $procedure = new StoredProcedure($this->connection, self::PROCEDURE_GERA_NOTA_CORPO_ENTRADA);
        $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT, $cdfilial,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIRECE',      Param::PARAM_INPUT, $cdfilirece,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDEMPRESA',       Param::PARAM_INPUT, $cdempresa,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDTIPOOPER',      Param::PARAM_INPUT, $cdtipooper,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERIEENTR',     Param::PARAM_INPUT, $cdserieentr,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLIENTE',       Param::PARAM_INPUT, $cdcliente,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTENTRSAID',      Param::PARAM_INPUT, $dtentrsaid,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDSISTEMA',       Param::PARAM_INPUT, $sistema,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRNOTAFISC',      Param::PARAM_INPUT, $vrnotafisc,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_PRXLANCTONF',     Param::PARAM_OUTPUT,$prxLanctoNf,       Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_PRXLANCESTQ',     Param::PARAM_OUTPUT,$prxLanctoEstoq,    Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_ATUAESTOQUE',     Param::PARAM_INPUT, $atualizaestoque,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAPURACFAT',     Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF',      Param::PARAM_INPUT, $nrlanctonf,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDTPIJURTRAN',    Param::PARAM_INPUT, $pesstrans,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRINSJURTRAN',    Param::PARAM_INPUT, $insctrans,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRPLACAVEIC',     Param::PARAM_INPUT, $placaveiculo,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SGUFVEIC',        Param::PARAM_INPUT, $ufveiculo,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDPGTOFRETE',     Param::PARAM_INPUT, $pgtofrete,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSQTDEVOLU',      Param::PARAM_INPUT, $quantidadedevolu,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSESPECVOLU',     Param::PARAM_INPUT, $espefvolu,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSMARCAVOLU',     Param::PARAM_INPUT, $marcvolu,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSNUMEVOLU',      Param::PARAM_INPUT, $numevolu,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPBRUTOVOLU',    Param::PARAM_INPUT, $vrbrutovolu,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPLIQUVOLU',     Param::PARAM_INPUT, $vrliquidovolu,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCESTQ',      Param::PARAM_INPUT, $nrlancestoq,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCORIG',      Param::PARAM_INPUT, $nrlancorig,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTLANCAMENNF',    Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRNOTAFISC',      Param::PARAM_INPUT, $nrnotafisc,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTEMISSAO',       Param::PARAM_INPUT, $dtemissao,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRACESSONFE',     Param::PARAM_INPUT, $nracessonfe,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDMODDOCFISC',    Param::PARAM_INPUT, $cdmododocfisc,     Param::PARAM_TYPE_STR));
        
        return $procedure->execute();
    }
    
    public function geraCorpoNota($cdfilial, $cdfiliemit, $cdempresa, $cdtipooper, $cdserie, $cdcliente, $sistema, $atualizaestoque, $nrlanctonf, $pesstrans, $insctrans, $placaveiculo, $ufveiculo, $pgtofrete, $quantidadedevolu, $espefvolu, $marcvolu, $numevolu, $nrlancestoq, $dtentrsaid, $vrbrutovolu, $vrliquidovolu, $nrlancorig = null, $dtlancmennf = null) {
        $dtentrsaid = str_replace("-", "/", $dtentrsaid);
        $cdtipooper = $cdtipooper['CDTIPOOPER'];
        
        $procedure = new StoredProcedure($this->connection, 'GERA_NOTAFISCAL_CORPO');
        $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT, $cdfilial,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIEMIT',      Param::PARAM_INPUT, $cdfiliemit,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDEMPRESA',       Param::PARAM_INPUT, $cdempresa,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDTIPOOPER',      Param::PARAM_INPUT, $cdtipooper,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERIE',         Param::PARAM_INPUT, $cdserie,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLIENTE',       Param::PARAM_INPUT, $cdcliente,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTENTRSAID',      Param::PARAM_INPUT, $dtentrsaid,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDSISTEMA',       Param::PARAM_INPUT, $sistema,           Param::PARAM_TYPE_STR));

        $procedure->addParam(new Param('P_PRXLANCTONF',     Param::PARAM_OUTPUT, NULL,              Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_PRXLANCESTQ',     Param::PARAM_OUTPUT, NULL,              Param::PARAM_TYPE_STR, 10));
        
        $procedure->addParam(new Param('P_ATUAESTOQUE',     Param::PARAM_INPUT, $atualizaestoque,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAPURACFAT',     Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF',      Param::PARAM_INPUT, $nrlanctonf,        Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDTPIJURTRAN',    Param::PARAM_INPUT, $pesstrans,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRINSJURTRAN',    Param::PARAM_INPUT, $insctrans,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRPLACAVEIC',     Param::PARAM_INPUT, $placaveiculo,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SGUFVEIC',        Param::PARAM_INPUT, $ufveiculo,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDPGTOFRETE',     Param::PARAM_INPUT, $pgtofrete,         Param::PARAM_TYPE_STR));
        
        
        $procedure->addParam(new Param('P_DSQTDEVOLU',      Param::PARAM_INPUT, $quantidadedevolu,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSESPECVOLU',     Param::PARAM_INPUT, $espefvolu,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSMARCAVOLU',     Param::PARAM_INPUT, $marcvolu,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSNUMEVOLU',      Param::PARAM_INPUT, $numevolu,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPBRUTOVOLU',    Param::PARAM_INPUT, $vrbrutovolu,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPLIQUVOLU',     Param::PARAM_INPUT, $vrliquidovolu,     Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_NRLANCESTQ',      Param::PARAM_INPUT, $nrlancestoq,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCORIG',      Param::PARAM_INPUT, $nrlancorig,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTLANCAMENNF',    Param::PARAM_INPUT, $dtlancmennf,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDGEROUTDOCU',    Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTCOMPFATUR',     Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPTOVNDFATURNF', Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        
        return $procedure->execute();
    }
    
    public function procedureGeraNotaFiscalProduto($filial, $numLancamento, $produto, $codEmissao, $descEmissao, $quantidade, $vrUnitario, $vrDesconto, $almoxarifado = null, $localEstoque = null, $nrloteestq = null, $nrsublote = null, $cdservfatu = null, $idOrigemProd = 'N', $idAtuEstProd = 'S',  $vrICMSItOrig = 0, $cdMarca = null) {
        
        $procedure = new StoredProcedure($this->connection, self::PROCEDURE_GERA_PRODUTO);
        
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $numLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODUTO', Param::PARAM_INPUT, $produto, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODEMIS', Param::PARAM_INPUT, $codEmissao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSPRODEMIS', Param::PARAM_INPUT, $descEmissao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_QTITEMPROD', Param::PARAM_INPUT, $quantidade, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRUNITITEM', Param::PARAM_INPUT, $vrUnitario, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRDESCITEM', Param::PARAM_INPUT, $vrDesconto, Param::PARAM_TYPE_STR));

        $procedure->addParam(new Param('P_CDFILIALITG', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRINTEGRACAO', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRITEMINTEGR', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCSTINFUSU', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCFOPINFUSU', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSUFXINFUSU', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));

        $procedure->addParam(new Param('P_CDALMOXARIFE', Param::PARAM_INPUT, $almoxarifado, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDLOCALESTOQ', Param::PARAM_INPUT, $localEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLOTEESTQ', Param::PARAM_INPUT, $nrloteestq, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSUBLOTE', Param::PARAM_INPUT, $nrsublote, Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDCONFORITEM', Param::PARAM_INPUT, 'S', Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDTPNFCREDEB', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_QTVOLUITEM', Param::PARAM_INPUT, 0, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAPUR', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDMARCA', Param::PARAM_INPUT, $cdMarca, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERVFATU', Param::PARAM_INPUT, $cdservfatu, Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDORIGEMPROD', Param::PARAM_INPUT, $idOrigemProd, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDATUESTPROD', Param::PARAM_INPUT, $idAtuEstProd, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRICMSSTITORIG', Param::PARAM_INPUT, $vrICMSItOrig, Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_DSJUSTIFICATIVA', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSINFADICPROD', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }
    
    private function callProcedureImpostoEntrada($empresa, $inscesta, $tipoOperacao, $fornecedor, $filial, $seqLancamento, $dtEntrada, $entrada_saida, $operador, $grupofiscal, $idinterest, $cdarvprod, $cdgrpfiscsv, $idorigdestnf, $filialRece, $cdfiliemit, $cliente, $valorProduto) {
            
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_IMPOSTO_ENTRADA_PRODUTO);
        $procedure->addParam(new Param('P_EMPRESA',         Param::PARAM_INPUT, $empresa,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INSCESTA',        Param::PARAM_INPUT, $inscesta,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TIPOOPER',        Param::PARAM_INPUT, $tipoOperacao,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFORNECED',      Param::PARAM_INPUT, $fornecedor,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILIAL',          Param::PARAM_INPUT, $filial,            Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LANCTONF',        Param::PARAM_INPUT, $seqLancamento,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INIVALID',        Param::PARAM_INPUT, $dtEntrada,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ENTRSAID',        Param::PARAM_INPUT, $entrada_saida,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR',      Param::PARAM_INPUT, $operador,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDGRUPOFISC',     Param::PARAM_INPUT, $grupofiscal,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDINTEREST',      Param::PARAM_INPUT, $idinterest,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDARVPROD',       Param::PARAM_INPUT, $cdarvprod,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDGRPFISCSV',     Param::PARAM_INPUT, $cdgrpfiscsv,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDORIGDESTNF',    Param::PARAM_INPUT, $idorigdestnf,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIRECE',      Param::PARAM_INPUT, $filialRece,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIEMIT',      Param::PARAM_INPUT, $cdfiliemit,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLIENTE',       Param::PARAM_INPUT, $cliente,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPRODUTO',       Param::PARAM_INPUT, $valorProduto,      Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }
    
   private function procedureGeraFechamentoEntrada($filial, $numLancamento, $numLancEstoque, $dtProcessamento, $operador = '') {
        $procedure = new StoredProcedure($this->connection, self::PROCEDURE_GERA_FECHAMENTO_ENTRADA);
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $numLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DATAPROC', Param::PARAM_INPUT, $dtProcessamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCESTQ', Param::PARAM_INPUT, $numLancEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $operador, Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    public function retornaSerieParaSaida($cdfilial) {
        try {
            $params = array(
                "P_CDFILIAL"  => $cdfilial
            );
            $query = "SELECT S.CDSERIE, IDCONMAXITEM, NRMAXITNF, S.CDEMPRESA
                        FROM SERIENF S, FILIAL F
                        WHERE F.CDFILIAL   = :P_CDFILIAL  AND
                              F.CDEMPRESA  = S.CDEMPRESA  AND
                              F.CDINSCESTA = S.CDINSCESTA AND
                              F.CDINSCMUNI = S.CDINSCMUNI AND
                              S.CDSERIE    = F.CDSERIE";
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
 
    public function gerarSaida($notasFiscais, $dadosDevolucao, $dadosAdicionais = array()) {
        $cdFilEmit  = $notasFiscais[0]["CDFILDEST"];
        $cdFilDest  = $notasFiscais[0]["CDFILDEST"];
        
        $dadosFilialOpLog   = $this->dfs48000SaidaDoOperadorLogisticoFactory->getDadosFilial($dadosDevolucao['CDFILIOPELOG']);
        $dadosFilialDest    = $this->dfs48000SaidaDoOperadorLogisticoFactory->getDadosFilial($cdFilDest);
        $paramsGeracao      = $this->dfs48000SaidaDoOperadorLogisticoFactory->getParamsGeracao($dadosDevolucao['CDFILIOPELOG']);
        
        $cdFilial           = $this->dfs48000SaidaDoOperadorLogisticoFactory->getCDFilial($cdFilDest);
        
        if(count($cdFilial)>0){
            $reserva = true;
        } else {
            $reserva = false;
        }
        
       
       
        
        $cdOperador = $this->environment->getCdOperador();
        $nmOperador = $this->dfs48000SaidaDoOperadorLogisticoFactory->getNmOperador($cdOperador);
        
        $nmFilial   = $dadosFilialOpLog["NMFILIAL"];
        $cdEmpresa  = $dadosFilialOpLog["CDEMPRESA"];
        $cdInScesta = $dadosFilialOpLog["CDINSCESTA"];
        $cdSerie    = empty($dadosDevolucao["CDSERIE"]) ? $dadosDevolucao["CDSERIE"] : $dadosFilialDest["CDSERIE"];
       
        if($cdSerie == NULL) {
            $retornaSerie = self::retornaSerieParaSaida($dadosDevolucao['CDFILIOPELOG']);
            
            if(!empty($retornaSerie)) {
                $cdSerie = $retornaSerie[0]['CDSERIE'];
            }
        }
          
        $organizacao = $this->environment->getNrOrgTrab();
        
        $geraNota = true;
        
        foreach($notasFiscais as $notaFiscal) {
            try {
                
                $produtos = $this->validaItensPedido($notaFiscal, $dadosDevolucao);
                 
                if(empty($produtos)) {
                    throw new \Exception(self::MSG_ERROS["LANC_N_GER"]);
                }
                
                $this->connection->beginTransaction();
                
                $nrLanctoNfNota = $this->NewCode->getCodeByTableName(self::NOVO_CODIGO_NOTA_LANC.$dadosDevolucao['CDFILIOPELOG'], $organizacao);    //LANCNF/FILIAL
                $nrLancEstoq    = $this->NewCode->getCodeByTableName(self::NOVO_CODIGO_LANC_ESTOQUE.$dadosDevolucao['CDFILIOPELOG'], $organizacao); //LANCTOEST
               
                $notaFiscal['NRLANCTONF'] = $nrLanctoNfNota;
                $notaFiscal['DTENTRSAID'] = $dadosDevolucao['DTENTREGA'];
                
                $proxLancamento = $this->geraCorpoNota($dadosDevolucao['CDFILIOPELOG'], $dadosDevolucao['CDFILIOPELOG'], $paramsGeracao["CDEMPRESA"], $dadosDevolucao["CDTIPOOPER"], $cdSerie, $cdFilEmit, 'DFS', 'S', $nrLanctoNfNota, '', '', '', '', $dadosDevolucao['FRETE']['CDFRETE'], '', '', '', '', $nrLancEstoq, $dadosDevolucao["DATA"], 0, 0, '', null);
                
                $this->insereDadosAdicionais($notaFiscal, $proxLancamento, $cdFilDest, $cdFilEmit, $dadosDevolucao, $organizacao, $paramsGeracao);
                
               
                
                $loopItem = 0;
                $vrTot = 0 ;
                
                foreach ($produtos as $produto) {
                    $loopItem += 1;
                    $cdProdEmis = $this->maskProduto($produto["CDARVPRODEST"], $organizacao);
                        
                    try {
                        //
                        if($reserva){
                            $erroReserva =  $this->dfs48000SaidaDoOperadorLogisticoFactory->erroReserva($cdFilDest,$notaFiscal["CDFILLANC"],$produto["NRAUTOFORN"], $produto["CDMARCA"],$produto["DTPROGENTAF"] , $produto["CDARVPROD"]);
                            if(count($erroReserva)==0){
                                 throw new \Exception('ERRORES');
                            }
                            
                        }
                        
                       $stWeb =  $this->getStWebService($produto);
                        if($produto["VRPREUNIAF"]<=0){
                             throw new \Exception('ERROVLR');
                        }
                        if(count($stWeb)>0 && $produto["QTATENDPEDOL"]<=0){
                             throw new \Exception('ERROQTD');
                        }
                          if($produto["QTESTOQDIA"]<=$produto["QTSAIDOPLOG"]){
                             throw new \Exception('ERROQTD');
                        }
                    //  
                        $adqVerifDuplNF = self::adqVerifDuplNF($dadosDevolucao['CDFILIOPELOG'], $cdFilDest, $this->environment->getCdFilial(), $produto);
                        
                        if(!empty($adqVerifDuplNF)) {
                            throw new \Exception('Nota Fiscal Duplicada.');
                        } else {
                            if($produto['IDUTILITEM'] == 1) {
                                 $adqNFRelacAFSai = self::adqNFRelacAFSai($dadosDevolucao['CDFILIOPELOG'], $nrLanctoNfNota, $cdFilDest, $this->environment->getCdFilial(), $produto);
                            
                                if(empty($adqNFRelacAFSai)) {
                                    self::acmInsNfRelacAfSai($dadosDevolucao['CDFILIOPELOG'], $this->environment->getCdFilial(), $nrLanctoNfNota, $cdFilDest, $produto);
                                }
                            } else {
                                if($dadosDevolucao['DATAENT'] == 0) {
                                    $adqNFRELACPEXOL = self::adqNFRELACPEXOL($dadosDevolucao['CDFILIOPELOG'], $this->environment->getCdFilial(), $nrLanctoNfNota, $cdFilDest, $produto);
                                
                                    if(empty($adqNFRELACPEXOL)) {
                                        self::acmInsNFRELACPEXOL($dadosDevolucao['CDFILIOPELOG'], $this->environment->getCdFilial(), $nrLanctoNfNota, $cdFilDest, $produto, null, $loopItem);
                                    }
                                } else {
                                    $adqExistItem = self::adqExistItem($cdFilDest, $this->environment->getCdFilial(), $produto);
                                    
                                    if(!empty($adqExistItem)) {
                                        $adqNFRELACPEXOL = self::adqNFRELACPEXOL($dadosDevolucao['CDFILIOPELOG'], $this->environment->getCdFilial(), $nrLanctoNfNota, $cdFilDest, $produto, $adqExistItem[0]['DTPROGENPEOL']);
                                    
                                        if(empty($adqNFRELACPEXOL)) {
                                            self::acmInsNFRELACPEXOL($dadosDevolucao['CDFILIOPELOG'], $this->environment->getCdFilial(), $nrLanctoNfNota, $cdFilDest, $produto, null, $loopItem);
                                        }
                                    } else {
                                        $adqNFRELACPEXOL = self::adqNFRELACPEXOL($dadosDevolucao['CDFILIOPELOG'], $this->environment->getCdFilial(), $nrLanctoNfNota, $cdFilDest, $produto);
                                        
                                        if(empty($adqNFRELACPEXOL)) {
                                            self::acmInsNFRELACPEXOL($dadosDevolucao['CDFILIOPELOG'], $this->environment->getCdFilial(), $nrLanctoNfNota, $cdFilDest, $produto, null, $loopItem);
                                        }
                                    }
                                }
                            }
                           
                            $this->procedureGeraNotaFiscalProduto($dadosDevolucao['CDFILIOPELOG'], $proxLancamento["P_PRXLANCTONF"], $produto["CDPRODUTOEST"], $cdProdEmis, $produto["NMPRODUTOEST"], $produto["QTPROGENTAF"], $produto["VRPREUNIAF"], 0, NULL, NULL, NULL, NULL, NULL, 'N', 'S', 0, $produto["CDMARCAOL"]);
                            //  $vrTot += $produto["VRPREUNIAF"] * $produto["QTPROGENTAF"];
                        }
                    } catch(\Exception $e) {
                       
                        $geraNota = false;
                        array_push($this->arrayProdutosErro, array(
                            "CDFILIOPELOG"  => $dadosDevolucao['CDFILIOPELOG'],
                            "NMFILIOPELOG"  => $nmFilial,
                            "CDFILDEST"     => $cdFilDest,
                            "NRLANCTONF"    => $notaFiscal["NRLANCTONF"],
                            "DTENTREGA"     => $notaFiscal["DTENTRSAID"],
                            "DTSAIDA"       => $dadosDevolucao["DATA"],
                            "CDARVPRODOL"   => $produto["CDARVPRODOL"],
                            "NMPRODUTOOL"   => $produto["NMPRODUTOOL"],
                            "CDARVPRODEST"  => $produto["CDARVPRODEST"],
                            "NMPRODUTOEST"  => $produto["NMPRODUTOEST"],
                            "DSERRO"        => $e->getMessage())
                        );
                    }
                }
             
                if($geraNota) {
                    // Realiza o Fechamento da nota
                    $this->lancamentoNotaFiscalService->procedureGeraFechamentoSaida($dadosDevolucao['CDFILIOPELOG'], $proxLancamento["P_PRXLANCTONF"], substr($paramsGeracao["DTPROCESSA"], 0, 10), $proxLancamento["P_PRXLANCESTQ"]);
                   
                    //Verifica se a operação atualiza estoque
                    if($this->operacaoAtualizaEstoque($dadosDevolucao["CDTIPOOPER"]["CDTIPOOPER"], 'S')) {
                        if($paramsGeracao["IDESTONLINE"] == 'S') {
                            // A função atualiza estoque também busca por irregularidades
                   
                         $produtosIrregulares = $this->validacoesEstoqueService->atualizaEstoque($dadosDevolucao['CDFILIOPELOG'], $proxLancamento["P_PRXLANCESTQ"]);
                        }
                    }
                    
                    $cdModDocFisc = $this->dfs48000SaidaDoOperadorLogisticoFactory->getCdModDocFisc($dadosDevolucao['CDFILIOPELOG'], $proxLancamento["P_PRXLANCTONF"]);
                    
                    if(empty($cdModDocFisc)) throw new \Exception(self::MSG_ERROS["SEMDOCUFISC"]);
                    
                    $this->verififcaImpostosObrigatorio($dadosDevolucao['CDFILIOPELOG'], $proxLancamento["P_PRXLANCTONF"]);

                    $this->geraRateio($dadosDevolucao['CDFILIOPELOG'], $proxLancamento["P_PRXLANCTONF"], 'S', '', $organizacao);
             
                    $this->connection->commit();
    
                    $vrTot = $this->dfs48000SaidaDoOperadorLogisticoFactory->getTotalSaida($dadosDevolucao['CDFILIOPELOG'], $proxLancamento["P_PRXLANCTONF"]);

                    array_push($this->arrayNotasSucesso, array(
                        "CDFILIOPELOG"  => $dadosDevolucao['CDFILIOPELOG'],
                        "NMFILIOPELOG"  => $nmFilial,
                        "CDFILDEST"     => $cdFilDest,
                        "NRLANCTONF"    => isset($notaFiscal["NRLANCTONF"]) ? $notaFiscal["NRLANCTONF"] : null,
                        "DTENTREGA"     => isset($notaFiscal["DTENTRSAID"]) ? $notaFiscal["DTENTRSAID"] : date("d/m/Y"),
                        "DTSAIDA"       => $dadosDevolucao["DATA"],
                        "DTENTRSAID"    => $dadosDevolucao["DATA"],
                        "DTEMISSAO"     => $dadosDevolucao["DATA"],
                        "CDSERIE"       => $cdSerie,
                        "VRNOTAFISC"    => $vrTot['VRNOTAFISC'],
                        "NRNOTAFISC"    => $vrTot['NRNOTAFISC'],
                        "USR_NMROTATRANS" => $notaFiscal['USR_NMROTATRANS']
                    ));
              
                } else {
                    array_push($this->arrayNotasErro, array(
                        "CDFILIOPELOG"  => $dadosDevolucao['CDFILIOPELOG'],
                        "NMFILIOPELOG"  => $nmFilial,
                        "CDFILDEST"     => $cdFilDest,
                        "NRLANCTONF"    => isset($notaFiscal["NRLANCTONF"]) ? $notaFiscal["NRLANCTONF"] : "Sem n° de lançamento para esta NF",
                        "DTENTREGA"     => isset($notaFiscal["DTENTRSAID"]) ? $notaFiscal["DTENTRSAID"] : "Sem Dt. Entrega para esta NF",
                        "DTSAIDA"       => $dadosDevolucao["DATA"],
                        "DSERRO"        => null,
                        "PROD_ERRO"     => $this->arrayProdutosErro)
                    );
                    $this->connection->rollback();

                }
            } catch(\Exception $e) {
            
                $this->connection->rollback();
                
                array_push($this->arrayNotasErro, array(
                    "CDFILIOPELOG"  => $dadosDevolucao['CDFILIOPELOG'],
                    "NMFILIOPELOG"  => $nmFilial,
                    "CDFILDEST"     => $cdFilDest,
                    "NRLANCTONF"    => isset($notaFiscal["NRLANCTONF"]) ? $notaFiscal["NRLANCTONF"] : "Sem n° de lançamento para esta NF",
                    "DTENTRSAID"    => isset($notaFiscal["DTENTRSAID"]) ? $notaFiscal["DTENTRSAID"] : "Sem Dt. Entrega para esta NF",
                    "DTSAIDA"       => $dadosDevolucao["DATA"],
                    "DSERRO"        => $e->getMessage(),
                    "PROD_ERRO"     => $this->arrayProdutosErro)
                );
            }
        }
        
        if($dadosDevolucao['IDUTRESPEDXOL'] == 'S') {
            self::acmDelReserva($dadosDevolucao['CDFILIOPELOG']);
        }
        
        self::gerarEntradaNova($dadosDevolucao['CDFILIOPELOG'], $cdFilDest, $dadosDevolucao["CDTIPOOPER"], $dadosDevolucao["DATA"], $this->arrayNotasSucesso,$dadosFilialDest);
        
        return array(
            'NFS_ERRO'    => $this->getErros(),
            'NFS_SUCESSO' => $this->arrayNotasSucesso,
            'NFE_ERRO'    => $this->arrayNFEErro,
            'NFE_SUCESSO' => $this->arrayNFESucesso,
            'SUCESSOS' => $this->notasSucesso,
        );
    }
    
    protected function getDadosGeracao($cdFilial) {
        $paramsGeracao = $this->dfs48000SaidaDoOperadorLogisticoFactory->getParamsGeracao($cdFilial);
        $paramSupri    = $this->dfs48000SaidaDoOperadorLogisticoFactory->getParamSupri($cdFilial);
        return array_merge($paramsGeracao, $paramSupri);
    }
    
    protected function getDataEntrada($cdFiliOperLog, $cdFilDest, $nrLanctonf, $dtEntrSaid) {
        $dataEmbarqueOl = $this->dfs48000SaidaDoOperadorLogisticoFactory->getDataEmbarqueOl($cdFiliOperLog, $cdFilDest, $nrLanctonf);
        if($dataEmbarqueOl) {
            if($dataEmbarqueOl[0]) {
                if($dataEmbarqueOl[0]['NRDIAEMBARQUE']) {
                    $nrDiaEmbarque = $dataEmbarqueOl[0]['NRDIAEMBARQUE'];
                    return date('d-m-Y', strtotime("+$nrDiaEmbarque days",strtotime($dtEntrSaid)));
                }
                else return $dtEntrSaid;
            }
            else return $dtEntrSaid;
        }
        else return $dtEntrSaid;
    }
    
    private function validaSerieExistente($cdSerie) {
        $serie = $this->Dfs48000EntradaDoOperadorLogisticoFactory->getSerieEntr($cdSerie);
        if(empty($serie)) throw new \Exception(self::MSG_ERROS["SERIE_ENTR"]);
    }
    
    private function validaNotaFiscalExistente($cdFilDest, $cdFiliOperLog, $nrNotaFisc) {
        $notaFiscal = $this->Dfs48000EntradaDoOperadorLogisticoFactory->getNotaFiscal($cdFilDest, $cdFiliOperLog, $nrNotaFisc);  
        if(isset($notaFiscal[0])) throw new \Exception(self::MSG_ERROS["LANC_N_GER"]);
    }
    
    protected function validaNotaEntradaOl($notaFiscal, $paramsGeracao, $cdFilDest, $cdFiliOperLog, $tipoOperEnt, $dtEntrada) {
        $this->validacoesParamCtrlFisOl($notaFiscal['DTENTRADA'], $paramsGeracao, $paramsGeracao["DTPROCESSA"], 
                                        $paramsGeracao["NRDIASENTRFIL"], $paramsGeracao["NRDIASENTR"], 
                                        $paramsGeracao["NRDIASENTRPD"], $paramsGeracao["NRDIASENTRPDFIL"]);  
        $this->validaSerieExistente($notaFiscal["CDSERIE"]);
        $this->validaDataIntegracao($paramsGeracao["CDEMPRESA"], 'E', $tipoOperEnt, $notaFiscal["DTENTRADA"]);
        $this->validaNotaFiscalExistente($cdFilDest, $cdFiliOperLog, $notaFiscal["NRLANCTONF"]);
        $this->validaValorNota($notaFiscal["VRNOTAFISC"]);
        $this->validaDataEmissao($notaFiscal["DTEMISSAO"], $dtEntrada);
        $this->validaNrNotaFisc($notaFiscal["NRLANCTONF"]);
    }
    

    public function gerarEntrada($cdfiliopelog, $cdfildest, $cdtipooper, $dtentrada, $notas, $dadosFilialDest) {
        $paramsGeracao = $this->getDadosGeracao($cdfildest);
        
        
        $tipoOperBol = $this->dfs48000SaidaDoOperadorLogisticoFactory->getTipoOperOpl($cdfiliopelog, $cdfildest);
        if(isset($tipoOperBol[0])) $tipoOperEnt = $tipoOperBol[0]["CDTIPOOPERENTR"];
        else $tipoOperEnt = $cdtipooper;
         
        $organizacao = $this->environment->getNrOrgTrab();
        $contador = 0;
        $CodigoNF = '';
        foreach($notas as $notaFiscal) {
            $this->prodImpos = array();
           $this->connection->beginTransaction();
            try {
                $notaFiscal['DTENTRADA'] = ($dtentrada) ? $dtentrada : $this->getDataEntrada($cdfiliopelog, $cdfildest, $notaFiscal['NRLANCTONF'], $notaFiscal['DTENTRSAID']);
              
                if($this->dfs48000SaidaDoOperadorLogisticoFactory->verificaNotaSendoUtilizada($cdfildest, $notaFiscal["NRLANCTONF"])) {
                    throw new \Exception('Operação bloqueada. A nota fiscal está sendo utilizada por outro operador!');
                }
                
                $feriFeli = $this->dfs48000SaidaDoOperadorLogisticoFactory->getFeriFeli($notaFiscal['DTENTRADA'], $cdfildest);
                
                $notaFiscal['DTENTRADA'] = str_replace("/", "-", $notaFiscal["DTENTRADA"]);
                
                while(!empty($this->dfs48000SaidaDoOperadorLogisticoFactory->getFeriFeli($notaFiscal['DTENTRADA'], $cdfildest)[0])) {
                    $notaFiscal['DTENTRADA'] = date('d-m-Y', strtotime("+1 days",strtotime($notaFiscal['DTENTRADA'])));
                }
                  
                $this->validaNotaEntradaOl($notaFiscal, $paramsGeracao, $cdfildest, $cdfiliopelog, $tipoOperEnt, $notaFiscal["DTENTRADA"]);
                
                $produtos = $this->dfs48000SaidaDoOperadorLogisticoFactory->buscaProdutosOrigem($cdfiliopelog, $notaFiscal["NRLANCTONF"], $cdfildest, $paramsGeracao["CDTIPCUSTEST"], $paramsGeracao["DTPROCESSAFILI"], $paramsGeracao["IDUTORIGPROD"]);
               foreach($produtos as $produto) {
                  $prodFili = $this->Dfs48000EntradaDoOperadorLogisticoFactory->adqProdfiliEntrada($produto['CDPRODUTO'],$cdfiliopelog);
                    
                    if(isset($prodFili[0]) && $prodFili[0]['IDPRODATIV'] =='A'){
                        if($produto['VRUNITITEM']>0 ){
                            if($contador == 0){
                                $nrLanctoNfNota = $this->NewCode->getCodeByTableName(self::NOVO_CODIGO_NOTA_LANC.$cdfildest, $organizacao);
                                $nrLancEstoq    = $this->NewCode->getCodeByTableName(self::NOVO_CODIGO_LANC_ESTOQUE.$cdfildest, $organizacao);
                                $tranpNf = $this->dfs48000SaidaDoOperadorLogisticoFactory->getTranpNf($notaFiscal["CDFILIOPELOG"], $notaFiscal["NRLANCTONF"]);
                                $CodigoNF = $this->geraCorpoNotaEntrada($cdfildest,$cdfildest, $dadosFilialDest["CDEMPRESA"],$tipoOperEnt,$notaFiscal["CDSERIE"],$cdfiliopelog, 'DFE', 'S',$nrLanctoNfNota,$tranpNf['IDTPIJURTRAN'], $tranpNf['NRINSJURTRAN'],'','',$tranpNf['IDPGTOFRETE'],'', '', '', '', $nrLancEstoq, $notaFiscal['DTENTRADA'],0,0,$notaFiscal["VRNOTAFISC"],'',0,$notaFiscal["DTEMISSAO"], $notaFiscal["NRNOTAFISC"] );
                                $this->dfs48000SaidaDoOperadorLogisticoFactory->updateNotaFisc($cdfildest,$CodigoNF['P_PRXLANCTONF'],$cdfiliopelog.$notaFiscal['NRLANCTONF'] );
                               if($notaFiscal['USR_NMROTATRANS'] != NULL) {
                                    $adqMaxDadosAdic = $this->adqMaxDadosAdic($cdfildest, $CodigoNF['P_PRXLANCTONF']);
                                    $this->dfs48000SaidaDoOperadorLogisticoFactory->insereDadosAdic($cdfildest,$CodigoNF['P_PRXLANCTONF'],$adqMaxDadosAdic[0]['NRDADOADIC']);
                               }
                            }
                            
                        $adqSugRegAux = $this->dfs48000SaidaDoOperadorLogisticoFactory->getSugRegEnt($cdfildest,$CodigoNF['P_PRXLANCTONF']);
                        if(!empty($adqSugRegAux)){
                            $adqSugRegSTEntr = $this->dfs48000SaidaDoOperadorLogisticoFactory->getSugRegSTEntr($cdfildest,$notaFiscal["NRLANCTONF"],$produto['NRSEQITPROD']);
                        }
                        $adqQtNFSaida =  $this->dfs48000SaidaDoOperadorLogisticoFactory->getQtNfSaida($cdfildest,$notaFiscal["NRLANCTONF"],$produto['CDPRODUTO'],$produto['CDMARCA']);
                        $dbQuantidade = $produto['QTNFRELAFSAI'];
                        $ftDescItemEntr = 0;
                        if(!empty($adqQtNFSaida)){
                            if(count($adqQtNFSaida)==1){
                                 $dbQuantidade = $adqQtNFSaida['QTITEMPROD'];
                                 $ftDescItemEntr = $adqQtNFSaida['VRDESCITEM']; 
                            } else {
                                // psquisar como fazer em php
                            }
                        }
                        
                        if($dbQuantidade>0){
                            $dbValorItem = $produto['VRUNITITEM'] * $dbQuantidade;
                            $dbValorUnitItem = $produto['VRUNITITEM'];
                            if(!empty($adqQtNFSaida)){
                                $dbValorItemSaida = $adqQtNFSaida['VRTOTAITEM'];
                                if (round($dbValorItem, 2) != round($dbValorItemSaida, 2) &&
                                          ($dbValorItem != $dbValorItemSaida) &&
                                          (Abs(dbValorItem - dbValorItemSaida) > 0.01)){ //transformar em php
                                              // valida se o total de saida está diferente do total de entrada
                                              // ERRO_TOT
                                    } 
                                }
                            if (!empty($adqSugRegSTEntr) || $adqSugRegAux['CDENCARGOST']==''){
                                $dbValorUnitItem = $produto['VRUNITITEM'];
                            }else{
                                $dbValorUnitItem = $produto['VRUNITITEM'] + $adqSugRegSTEntr['VRIMPOSTO'];
                                }
                            $dbValorItem = $dbQuantidade * $dbValorUnitItem;   
                            if($produto['IDUTILITEM']=='1'){
                                $adqNFRelacAF =  $this->dfs48000SaidaDoOperadorLogisticoFactory->getNFRelacAF($cdfildest,$produto['CDFILLANC'],$CodigoNF['P_PRXLANCTONF'],$produto['CDFILDEST'],$produto['NRAUTOFORN'],$produto['CDPRODUTO'],$produto['CDMARCA'],$produto['DTPROGENTAF']);

                            }
                                
                                
                                
                            }
                        }
                    }
               }
            } catch(\Exception $e) {
             
                $this->connection->rollback();
                
                array_push($this->arrayNotasErroEntrada, array(
                        "CDFILIAL"   => '',
                        "NRLANCTONF" => $notaFiscal["NRLANCTONF"],
                        "DTENTRSAID" => $notaFiscal["DTENTRSAID"],
                        "DTENTRADA"  => $notaFiscal["DTENTRADA"],
                        "NMPRODUTO"  => '',
                        "DSERRO"     => $e->getMessage()
                    )
                );
            }
        }
    }
    
    public function gerarEntradaNova($cdFiliOperLog, $cdFilDest, $cdTipoOper, $dtEntrada, $notasFiscais) {
    // Obtém parâmetros de geração
    $paramsGeracao = $this->getDadosGeracao($cdFilDest);

    // Obtém o tipo de operação
    $tipoOperBol = $this->dfs48000SaidaDoOperadorLogisticoFactory->getTipoOperOpl($cdFiliOperLog, $cdFilDest);
    $tipoOperEnt = isset($tipoOperBol[0]) ? $tipoOperBol[0]["CDTIPOOPERENTR"] : $cdTipoOper;

    // Obtém a organização
    $organizacao = $this->environment->getNrOrgTrab();

    // Inicializa o contador de sucesso
    $this->notasSucesso = 0;

    // Processa cada nota fiscal
    foreach($notasFiscais as $notaFiscal) {
        $this->prodImpos = array();
        try {
            $this->connection->beginTransaction();

            // Define a data de entrada
            $notaFiscal['DTENTRADA'] = $dtEntrada ?: $this->getDataEntrada($cdFiliOperLog, $cdFilDest, $notaFiscal['NRLANCTONF'], $notaFiscal['DTENTRSAID']);
            
            // Verifica se a nota está sendo utilizada
            if($this->dfs48000SaidaDoOperadorLogisticoFactory->verificaNotaSendoUtilizada($cdFilDest, $notaFiscal["NRNOTAFISC"])) {
                throw new \Exception('Operação bloqueada. A nota fiscal está sendo utilizada por outro operador!');
            }

            // Ajusta a data de entrada se for feriado
            $notaFiscal['DTENTRADA'] = str_replace("/", "-", $notaFiscal["DTENTRADA"]);
            while(!empty($this->dfs48000SaidaDoOperadorLogisticoFactory->getFeriFeli($notaFiscal['DTENTRADA'], $cdFilDest)[0])) {
                $notaFiscal['DTENTRADA'] = date('d-m-Y', strtotime("+1 day", strtotime($notaFiscal['DTENTRADA'])));
            }

            // Valida a nota fiscal
            $this->validaNotaEntradaOl($notaFiscal, $paramsGeracao, $cdFilDest, $cdFiliOperLog, $tipoOperEnt, $notaFiscal["DTENTRADA"]);

            // Busca os produtos e transporte
            // $produtos = $this->dfs48000SaidaDoOperadorLogisticoFactory->buscaProdutosOrigem($cdFiliOperLog, $notaFiscal["NRLANCTONF"], $cdFilDest, $paramsGeracao["CDTIPCUSTEST"], $paramsGeracao["DTPROCESSAFILI"], $paramsGeracao["IDUTORIGPROD"]);
          $produtos = $this->Dfs48000EntradaDoOperadorLogisticoFactory->buscaProdutosOrigem($cdFiliOperLog, $notaFiscal["NRLANCTONF"], $cdFilDest, $paramsGeracao["CDTIPCUSTEST"], $paramsGeracao["DTPROCESSAFILI"], $paramsGeracao["IDUTORIGPROD"]);
          
            $tranpNf = $this->dfs48000SaidaDoOperadorLogisticoFactory->getTranpNf($notaFiscal["CDFILIOPELOG"], $notaFiscal["NRLANCTONF"]);
           
            // Gera novos códigos
            $nrLanctoNfNota = $this->NewCode->getCodeByTableName(self::NOVO_CODIGO_NOTA_LANC.$cdFilDest, $organizacao);
            $nrLancEstoq    = $this->NewCode->getCodeByTableName(self::NOVO_CODIGO_LANC_ESTOQUE.$cdFilDest, $organizacao);

            // Gera a nota fiscal de entrada
            $proxLancamento = $this->geraCorpoNotaEntrada($cdFilDest, $cdFilDest, $paramsGeracao["CDEMPRESA"], $tipoOperEnt, $notaFiscal["CDSERIE"], $cdFiliOperLog, 'DFE',
                'S', $nrLanctoNfNota, $tranpNf["IDTPIJURTRAN"], $tranpNf["NRINSJURTRAN"], '', '', $tranpNf["IDPGTOFRETE"],
                '', '', '', '', $nrLancEstoq, $notaFiscal["DTENTRADA"], 0, 0, $notaFiscal["VRNOTAFISC"], '', $notaFiscal["DTEMISSAO"], $notaFiscal["NRNOTAFISC"], '', '');

            $nrAcessoNfe = $this->dfs48000SaidaDoOperadorLogisticoFactory->getNrAcessoNfe($cdFiliOperLog, $notaFiscal["NRLANCTONF"]);
         
            $this->dfs48000SaidaDoOperadorLogisticoFactory->setNrAcessoNfe($cdFilDest, $nrLanctoNfNota, $nrAcessoNfe, $cdFilDest.$notaFiscal["NRLANCTONF"]);

            $sugRegAux = $this->dfs48000SaidaDoOperadorLogisticoFactory->getSugRegEnt($cdFilDest, $nrLanctoNfNota);

            // Inicializa variáveis para controle
            $geraNota = true;
            $nrSeqitpeol = 1;
            $contadorProd = 1;
            // Processa cada produto
            foreach($produtos as $produto) {
                try {
                    // Valida o produto
                    $this->validaProduto($produto, $notaFiscal, $notaFiscal["CDFILDEST"], $sugRegAux, $cdFilDest);
                    $quantidadeItem = $this->getQuantidadeItemNota($notaFiscal["CDFILDEST"], $notaFiscal["NRLANCTONF"], $produto["CDPRODUTO"], $produto["CDMARCA"], $produto);

                    // Calcula o valor unitário do item
                    $valorUniItem = isset($sugRegAux[0]) ? ($sugRegAux["CDENCARGOST"] ? $produto["VRUNITITEM"] : $produto["VRUNITITEM"] + $this->dfs48000SaidaDoOperadorLogisticoFactory->getSugRegEntr($notaFiscal["CDFILDEST"], $notaFiscal["NRLANCTONF"], $produto["NRSEQITPROD"])["VRIMPOSTO"]) : $produto["VRUNITITEM"];
                    $valorItem = $valorUniItem * $quantidadeItem;

                    // Trata notas relacionadas a AF
                    if($produto["IDUTILITEM"] == '1') {
                        $nfRelacAf = $this->dfs48000SaidaDoOperadorLogisticoFactory->getNfRelacAf($cdFilDest, $produto["CDFILLANC"], $proxLancamento["P_PRXLANCTONF"], $produto["NRAUTOFORN"], $produto["CDPRODUTO"], $produto["CDMARCA"], $produto["DTPROGENTAF"]);
                        if(!isset($nfRelacAf[0])) {
                            $this->dfs48000SaidaDoOperadorLogisticoFactory->insereNfRelacAf($cdFilDest, $produto["CDFILLANC"], $proxLancamento["P_PRXLANCTONF"], $cdFilDest, $produto["NRAUTOFORN"], $produto["CDPRODUTO"], $produto["CDMARCA"], $produto["DTPROGENTAF"], $produto["QTNFRELAFSAI"], $valorUniItem, $organizacao);
                        }
                        $nfRelacAf = $this->dfs48000SaidaDoOperadorLogisticoFactory->getNfRelacAf($cdFilDest, $produto["CDFILLANC"], $proxLancamento["P_PRXLANCTONF"], $produto["NRAUTOFORN"], $produto["CDPRODUTO"], $produto["CDMARCA"], $produto["DTPROGENTAF"]);
                       
                        $this->dfs48000SaidaDoOperadorLogisticoFactory->itBaixaAf($produto["CDFILDEST"], $produto["CDFILLANC"], $produto["NRAUTOFORN"], $produto["CDPRODUTO"], $produto["CDMARCA"], $produto["DTPROGENTAF"]);
                        $this->dfs48000SaidaDoOperadorLogisticoFactory->baixaAf($produto["CDFILDEST"], $produto["CDFILLANC"], $produto["NRAUTOFORN"]);
                    } else {
                        // Relaciona nota
                        $relacNfOl = $this->dfs48000SaidaDoOperadorLogisticoFactory->relacNfOl($notaFiscal["CDFILIOPELOG"], $notaFiscal["NRLANCTONF"], $produto["CDPRODUTO"], $produto["CDMARCA"]);
                            
                        foreach ($relacNfOl as $relac) {
                            $relacNfOlProd = $this->dfs48000SaidaDoOperadorLogisticoFactory->relacNfOlProd($cdFilDest, $proxLancamento["P_PRXLANCTONF"], $relac["CDFILDEST"], $relac["CDFILLANC"], $relac["NRPEDEXOL"], $relac["CDMARCA"], $relac["CDPRODUTO"], $relac["DTPROGENPEOL"]);
                           
                            if(!isset($relacNfOlProd[0])) {
                                
                                $this->dfs48000SaidaDoOperadorLogisticoFactory->insereNfRelacOlProd($cdFilDest, $relac["CDFILLANC"], $proxLancamento["P_PRXLANCTONF"], $relac["CDFILDEST"], $relac["NRPEDEXOL"], $relac["CDPRODUTO"], $relac["CDMARCA"], $relac["DTPROGENPEOL"], $quantidadeItem, $valorUniItem,str_pad($nrSeqitpeol,4,'0',STR_PAD_LEFT) , $organizacao);
                              
                                $nrSeqitpeol++;
                            } else {
                                $this->dfs48000SaidaDoOperadorLogisticoFactory->updateNfRelacOlProd($cdFilDest, $relac["CDFILLANC"], $proxLancamento["P_PRXLANCTONF"], $relac["CDFILDEST"], $relac["NRPEDEXOL"], $relac["CDPRODUTO"], $relac["CDMARCA"], $relac["DTPROGENPEOL"]);
                            }
                            
                            $this->dfs48000SaidaDoOperadorLogisticoFactory->baixaItemOl($relac["CDFILDEST"], $relac["CDFILLANC"], $relac["NRPEDEXOL"], $relac["CDPRODUTO"], $relac["CDMARCA"], $relac["DTPROGENPEOL"]);
                        
                            $this->dfs48000SaidaDoOperadorLogisticoFactory->baixaPedOl($relac["CDFILDEST"], $relac["CDFILLANC"], $relac["NRPEDEXOL"]);
                          
                            
                        }
                    }

                    // Calcula ICMS ST se aplicável
                    $vrICMSST = isset($sugRegEntr[0]) ? ($sugRegEntr["VRIMPOSTO"] * $quantidadeItem) : 0;

                    // Gera nota fiscal para o produto
                    $cdProdEmis = $this->maskProduto($produto["CDARVPROD"], $organizacao);
                    $this->procedureGeraNotaFiscalProduto($cdFilDest, $proxLancamento["P_PRXLANCTONF"], $produto["CDPRODUTO"], $cdProdEmis, $produto["NMPRODUTO"], $quantidadeItem, $valorUniItem, 0, '', '', '', '', '', $produto["IDORIGEMPROD"], 'S', $vrICMSST, $produto["CDMARCA"]);

                    // Cria impostos associados
               
                    $this->criaImpostos($cdFilDest, $proxLancamento["P_PRXLANCTONF"], $tipoOperEnt, $cdFiliOperLog, $notaFiscal["DTENTRADA"], $valorItem, $contadorProd, $produto["IDORIGEMPROD"], $paramsGeracao["CDEMPRESA"], $produto["CDARVPROD"], $paramsGeracao["CDINSCESTA"], $organizacao);

                    // Atualiza o status da AF e nota
                    $seqItProdFormatado = str_pad($contadorProd, 4, "0", STR_PAD_LEFT);
                  
                    $erro = $this->callBaixaAFNf($cdFilDest, $proxLancamento["P_PRXLANCTONF"], $seqItProdFormatado, 'I', $cdProdEmis, $cdProdEmis, $produto["CDPRODUTO"], $produto["CDPRODUTO"], $produto["CDMARCA"], $produto["CDMARCA"], $quantidadeItem, $quantidadeItem);
                    
                    if (isset($erro) && $erro !== 'N') {
                        throw new \Exception($erro);
                    }

                    $this->dfs48000SaidaDoOperadorLogisticoFactory->updateItAfRelac($produto["CDFILDEST"], $produto["CDFILLANC"], $produto["NRAUTOFORN"], $produto["CDPRODUTO"], $produto["CDMARCA"], $produto["DTPROGENTAF"]);

                    if ($produto["IDUTILITEM"] == '1') {
                        $this->dfs48000SaidaDoOperadorLogisticoFactory->updateNrAutoFornProd($produto["NRAUTOFORN"], $cdFilDest, $proxLancamento["P_PRXLANCTONF"]);
                    }

                    $contadorProd++;
                } catch (\Exception $e) {
                    $geraNota = false;
                    $this->arrayProdutosErro[] = [
                        "CDFILIAL" => $cdFilDest,
                        "NRLANCTONF" => $notaFiscal["NRLANCTONF"],
                        "DTENTRSAID" => $notaFiscal["DTENTRSAID"],
                        "DTENTRADA" => $notaFiscal["DTENTRADA"],
                        "NMPRODUTO" => $produto["NMPRODUTO"],
                        "DSERRO" => $e->getMessage()
                    ];
                }
            }
                try{
            if ($geraNota) {
              $this->procedureGeraFechamentoEntrada($cdFilDest, $proxLancamento["P_PRXLANCTONF"], $proxLancamento["P_PRXLANCESTQ"], $paramsGeracao["DTPROCESSA"], $this->environment->getCdOperador());
                    
                    $lanctoestoq = Lanctoestoq::findOneByCdfilialAndNrlancestqAndNrorg($cdFilDest, $proxLancamento["P_PRXLANCESTQ"], $this->environment->getNrOrgTrab());
                     // NÃO COMENTAR ESTA LINHA EM HIPOTESE ALGUMA
                    // CASO NECESSITE DE ALTERAÇÕES PROCURAR PEDRO MOUTINHO OU RAFAEL DIAS
                    if (is_object($lanctoestoq))
                         $irregularidades = $this->verificaIrregularidades($lanctoestoq);
                         
             

                    $this->geraRateioEntradaOpl($cdFilDest, $proxLancamento["P_PRXLANCTONF"], 'E', '');
                    $paramSupri = $this->dfs48000SaidaDoOperadorLogisticoFactory->getParamSupri($cdFilDest);
                    if($paramSupri["IDUTCTRLFISOL"] == 'S') {
                        
                        $utControlFisc = $this->utilizaControleFiscal($cdFilDest, 'E', $paramsGeracao);
                        if($utControlFisc) $this->verififcaImpostoObrigatorio($cdFilDest, $proxLancamento["P_PRXLANCTONF"]);
                        
                        $qtItensEntrada = $this->dfs48000SaidaDoOperadorLogisticoFactory->getQtItemNotaEntrada($cdFilDest, $proxLancamento["P_PRXLANCTONF"]);
                        $qtItensSaida   = $this->dfs48000SaidaDoOperadorLogisticoFactory->getQtItemNotaSaida($notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"]);
                        if($qtItensEntrada !== $qtItensSaida) throw new \Exception(self::MSG_ERROS["ERRO_ITM"]);
                        
                        $vlrNotaEntrada = $this->dfs48000SaidaDoOperadorLogisticoFactory->getValorNotaEntrada($cdFilDest, $proxLancamento["P_PRXLANCTONF"]);
                        $vlrNotaSaida   = $this->dfs48000SaidaDoOperadorLogisticoFactory->getValorNotaSaida($notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"]);
                        
                        if($vlrNotaEntrada !== $vlrNotaSaida) throw new \Exception(self::MSG_ERROS["ERRO_TOT"]);

                    }
                    $this->connection->commit();
                    $this->notasSucesso++;
                    //$this->connection->rollback();
       
                
            } else {
                $this->connection->rollBack();
            }
            } catch (\Exception $e) {
            $this->connection->rollBack();
            $this->arrayNotasErro[] = [
                "CDFILIAL" => $cdFilDest,
                "NRLANCTONF" => $notaFiscal["NRLANCTONF"],
                "DSERRO" => $e->getMessage()
            ];
        }
        } catch (\Exception $e) {
            $this->connection->rollBack();
            $this->arrayNotasErro[] = [
                "CDFILIAL" => $cdFilDest,
                "NRLANCTONF" => $notaFiscal["NRLANCTONF"],
                "DSERRO" => $e->getMessage()
            ];
        }
    }
}

public function verificaIrregularidades(Lanctoestoq $lanctoestoq, $seqItemCodes = null) {
        $concat_cdarvprod_irregular_products = '';
        
        #checks for irregularities
        $irregularProducts = $this->retornaProdutosIrregulares($lanctoestoq, $seqItemCodes);
        
        foreach($irregularProducts as $produtos){
            $entitiesIrregularProducts = Produto::findOneByCdprodutoAndNrorg($produtos->getCdproduto(), $this->environment->getNrOrgTrab());
            $concat_cdarvprod_irregular_products .= $entitiesIrregularProducts->getCdarvprod().'. ';
        }
        
        #TODO Verify a better message
        if (!empty($irregularProducts))
            throw new \Exception ('A confirmação da operação faria com que a posição de estoque de alguns itens se tornasse irregular. Código dos produto(s): ' . $concat_cdarvprod_irregular_products);
    }
    
    public function retornaProdutosIrregulares(Lanctoestoq $lanctoestoq, $seqItens = null) {
        $nrmovilote = $lanctoestoq->getCdfilial().$lanctoestoq->getNrlancestq();

        $produtosIrregulares = Movilote::findByNrmoviloteAndNrorg(
            $nrmovilote, $lanctoestoq->getNrorg()
        );
        
        if(!is_null($seqItens)) {
            $itlanctoest = $this->buscaItlanctoest(
                $lanctoestoq->getCdfilial(), $lanctoestoq->getNrlancestq(),
                $lanctoestoq->getNrorg()
            );

            $seqProds = array();
            foreach($itlanctoest as $item)
                if(in_array($item->getNrsequitem(), $seqItens))
                    $seqProds[] = $item->getCdproduto();

            $produtosIrregularesInSeqItens = array();

            foreach ($produtosIrregulares as $produto)
                if(in_array($produto->getCdproduto(), $seqProds))
                    $produtosIrregularesInSeqItens[] = $produto;

            return $produtosIrregularesInSeqItens;
        }

        /** @var \Estoque\Entities\Movilote $produtosIrregulares */
        return $produtosIrregulares;
    }

protected function verififcaImpostoObrigatorio($cdFilial, $nrLanctonf) {
        $notaFiscal   = $this->dfs48000SaidaDoOperadorLogisticoFactory->getDadosNf($cdFilial, $nrLanctonf);
        $impostosObri = $this->dfs48000SaidaDoOperadorLogisticoFactory->getImpostoObrigatorio($notaFiscal["IDENTRSAIDOP"], $notaFiscal["CDTIPOOPER"]);
        $impostosErro = '';
        $produtosComErro = false;

        foreach ($impostosObri as $impObri) {
            if($impObri["IDPRODNOTA"] == 'P') {
                $impostoProdutos = $this->dfs48000SaidaDoOperadorLogisticoFactory->getImpostoProdutos($cdFilial, $nrLanctonf, $impObri["CDIMPOSTO"]);

                foreach ($impostoProdutos as $impostoProduto) {
                    $produtosComErro = true;
                    array_push($this->arrayImpostosErro, array("NMPRODUTO"  => $impostoProduto["NMPRODUTO"],
                                                         "DSERRO"     => "Produto sem incidência de imposto obrigatório. Imposto: ".$impObri["SGIMPOSTO"]));
                }
            }
            else {
                $impostosCorpo   = $this->dfs48000SaidaDoOperadorLogisticoFactory->getImpostoNota($cdFilial, $nrLanctonf, $impObri["CDIMPOSTO"]);
                if(!isset($impostosCorpo[0])) {
                    $notaErro = true;   
                    $impostosErro.= $impObri["SGIMPOSTO"].', ';
                }
                else if($impostosCorpo[0]["CDFILIAL"] == null) {
                    $notaErro = true;   
                    $impostosErro.= $impObri["SGIMPOSTO"].', ';
                }
            }
        }
        
        if($produtosComErro) throw new \Exception("Notafiscal possui produtos sem incidência de imposto(s) obrigatório(s).");
        $size = strlen($impostosErro);

        if($size != 0) {
            $impostosErro = substr($impostosErro,0, $size-2);
            throw new \Exception("Nota Fiscal sem incidência do(s) imposto(s) obrigatório(s) ".$impostosErro.'.');
        }
        
    }
    
protected function utilizaControleFiscal($cdFilial, $idEntrSaidOp, $paramsGeracao) {
        if($idEntrSaidOp === 'E') {
            if($paramsGeracao['IDUTCTRLFISENTRPG'] === 'S' || $paramsGeracao['IDUTCTRLFISENTRPF'] === 'S') return true;
            else return true;
        }
        else if($idEntrSaidOp === 'S') {
            if($paramsGeracao['IDUTCTRLFISSAIDPG'] === 'S' || $paramsGeracao['IDUTCTRLFISSAIDPF'] === 'S') return true;
            else return false;
        }
        else return false;
    }

  protected function geraRateioEntradaOpl($cdFilial, $nrLanctonf, $idEntrSaid, $rateioNf, $autMan = 'N', $realizaManut = 'P') {
        if($realizaManut != 'S') {
            if($realizaManut == 'P') {
                $this->dfs48000SaidaDoOperadorLogisticoFactory->deleteRateioNf($cdFilial, $nrLanctonf);
                $notaFiscal = $this->dfs48000SaidaDoOperadorLogisticoFactory->getNotaFiscalRateio($cdFilial, $nrLanctonf);
                $consContaRateio = $this->dfs48000SaidaDoOperadorLogisticoFactory->getConsContaRateio($idEntrSaid, $notaFiscal["CDTIPOOPER"]);
                
                if($idEntrSaid == 'E') {
                    if($consContaRateio["IDCONSCTBRAT"] == 'S') {
                        $contaPagar = $this->dfs48000SaidaDoOperadorLogisticoFactory->getContaPagar($cdFilial, $nrLanctonf);
                        if(isset($contaPagar[0])) {
                            $tipoConta = $contaPagar[0]["CDTPCTPAGAR"];
                        }
                        else {
                            $tipoConta = $this->dfs48000SaidaDoOperadorLogisticoFactory->getTpContaNfEntr($cdFilial, $nrLanctonf);
                        }
                        
                        $produtos     = $this->dfs48000SaidaDoOperadorLogisticoFactory->getItemProdutoRateio($cdFilial, $nrLanctonf);
                        
                        $rateioCtEntr = $this->dfs48000SaidaDoOperadorLogisticoFactory->getRatTpCtpr($tipoConta);
                        
                        foreach ($produtos as $produto) {
                            foreach ($rateioCtEntr as $rat) {
                                $filiRateio = ($rat["CDFILIAL"]) ? $rat["CDFILIAL"] : $notaFiscal["CDFILIRECE"];
                                
                                $rateioProd = $this->dfs48000SaidaDoOperadorLogisticoFactory->getRateioProd($cdFilial, $nrLanctonf, $produto["NRSEQITPROD"], $filiRateio, $rat["CDCENTCUST"]);
                                
                                $vlrRateio  = $produto["VRCONTITEM"] * ($rat["VRPERATTPCTP"] / 100);
                                
                                if($rateioProd[0]) $this->dfs48000SaidaDoOperadorLogisticoFactory->updateRateioProd($cdFilial, $nrLanctonf, $produto["NRSEQITPROD"], $filiRateio, $vlrRateio, $rat["CDCENTCUST"], $rat["VRPERATTPCTP"]);
                                else $this->dfs48000SaidaDoOperadorLogisticoFactory->insereRateioProd($cdFilial, $nrLanctonf, $produto["NRSEQITPROD"], $filiRateio, $rat["CDCENTCUST"], $vlrRateio, $rat["VRPERATTPCTP"], $organizacao);
                            }
                        }
                        
                    }
                    else {
                        $this->dfs48000SaidaDoOperadorLogisticoFactory->insereRateioNfeEntr($cdFilial, $nrLanctonf, $notaFiscal["CDFILIRECE"]);
                    }
                }
            }
        }
    }
    
//  protected function validaProduto($produto, $notaFiscal, $cdFilial, $sugRegAux, $cdFilDest) {
//         $this->validaProdutoPorFilial($cdFilDest, $produto["CDPRODUTO"]);
//         $this->validaValorProduto($produto["VRUNITITEM"]);
//         $this->validaQuantidadeProd($cdFilial, $notaFiscal["NRLANCTONF"], $produto["CDPRODUTO"], $produto["CDMARCA"], $produto);
//     }
    private function validaProdutoPorFilial($cdFilDest, $cdProduto) {
        $prodFili = $this->dfs48000SaidaDoOperadorLogisticoFactory->getProdFili($cdFilDest, $cdProduto); 
      
        if(isset($prodFili)) {
            if($prodFili['IDPRODATIV'] !== 'A') throw new \Exception(self::MSG_ERROS["PRODUTO_INATIVO"]); 
        }
        else throw new \Exception(self::MSG_ERROS["PRODUTO_FILIAL"]);
    }
    //  private function validaValorProduto($valorProduto) {
    //     if(!($valorProduto > 0)) throw new \Exception(self::MSG_ERROS["PRECO_ZERADO"]);
    // }
    
    private function validaQuantidadeProd($cdFilial, $nrLanctonf, $cdProduto, $cdMarca, $produto) {
        $quantidadeItem = $this->getQuantidadeItemNota($cdFilial, $nrLanctonf, $cdProduto, $cdMarca, $produto);
        if(!isset($quantidadeItem)) throw new \Exception(self::MSG_ERROS["QTDE_ZERADA"]);
    }
    
    private function getQuantidadeItemNota($cdFilial, $nrLanctonf, $cdProduto, $cdMarca, $produto) {
        $qtItemNota = $this->dfs48000SaidaDoOperadorLogisticoFactory->getQtItemNota($cdFilial, $nrLanctonf, $cdProduto, $cdMarca);
        $quantidadeItem = '';                   
        if(count($qtItemNota) !== 0){
            if(count($qtItemNota) == 1) $quantidadeItem = $qtItemNota[0]["QTITEMPROD"];
            else{
                foreach ($qtItemNota as $qtProd) {
                    if($qtProd == $produto["QTNFRELAFSAI"]) $quantidadeItem = $qtProd;
                }
            }
        } 
        else {
            $quantidadeItem = $produto["QTNFRELAFSAI"];
        } 
        return $quantidadeItem;
    }
    
    public function callConsultaGrupofiscal($idorigdestnf, $fornecedor, $filial, $entrada_saida, $tipoOperacao) {
        $procedure = new StoredProcedure($this->connection, self::PROCEDURE_CONSULTA_GRUPOFISCAL);
        $procedure->addParam(new Param("P_IDORIGDESTNF", Param::PARAM_INPUT, $idorigdestnf, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDORIGDESTNF", Param::PARAM_INPUT, $fornecedor, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDFILIFATUR", Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_IDENTRSAIDOP", Param::PARAM_INPUT, $entrada_saida, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDTIPOOPER", Param::PARAM_INPUT, $tipoOperacao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDGRUPOFISC", Param::PARAM_OUTPUT, '', Param::PARAM_TYPE_STR, 10));
        $output = $procedure->execute();
       
        return $output['P_CDGRUPOFISC'];
    }
    
     public function callBaixaAFNf($cdFilial, $nrLanctonf, $nrSeqItProd, $idStatus, $newCdProdEmis, $oldCdProdEmis, $newCdProduto, $oldCdProduto,
                $newCdMarca, $oldCdMarca, $newQtItProd, $oldQtItProd) {
        $procedure = new StoredProcedure($this->connection, self::PROCEDURE_BAIXA_AF);
        
        $procedure->addParam(new Param("P_CDFILIAL",          Param::PARAM_INPUT,  $cdFilial,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_NRLANCTONF",        Param::PARAM_INPUT,  $nrLanctonf,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_NRSEQITPROD",       Param::PARAM_INPUT,  $nrSeqItProd,            Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_IDSTATUS",          Param::PARAM_INPUT,  $idStatus,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_NEWCDPRODEMIS",     Param::PARAM_INPUT,  $newCdProdEmis,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_OLDCDPRODEMIS",     Param::PARAM_INPUT,  $oldCdProdEmis,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_NEWCDPRODUTO",      Param::PARAM_INPUT,  $newCdProduto,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_OLDCDPRODUTO",      Param::PARAM_INPUT,  $oldCdProduto,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_NEWCDMARCA",        Param::PARAM_INPUT,  $newCdMarca,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_OLDCDMARCA",        Param::PARAM_INPUT,  $oldCdMarca,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_NEWQTITPROD",       Param::PARAM_INPUT,  $newQtItProd,            Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_OLDQTITPROD",       Param::PARAM_INPUT,  $oldQtItProd,            Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_IDERRO",            Param::PARAM_OUTPUT, '',                      Param::PARAM_TYPE_STR, 10));
        
        $output = $procedure->execute();

        return $output['P_IDERRO'];
    }
    
    
    public function callConsultaIncidenciaImpostos($entrSaidIn, $cdEmpresa, $cdArvProd, $cdInscEsta, $cdTipoOper, $cdFilial, $nrLanctonf, $cdGrupoFisc, $interEstad, $origemProd) {
        
        $procedure = new StoredProcedure($this->connection, self::PROCEDURE_CONSULTA_INCIDENCIA_IMP);
        
        $procedure->addParam(new Param("P_ENTRSAIDIN",      Param::PARAM_INPUT, $entrSaidIn,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_EMPRESA",         Param::PARAM_INPUT, $cdEmpresa,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_PRODUTO",         Param::PARAM_INPUT, $cdArvProd,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_INSCESTA",        Param::PARAM_INPUT, $cdInscEsta,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_TIPOOPER",        Param::PARAM_INPUT, $cdTipoOper,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_FILIAL",          Param::PARAM_INPUT, $cdFilial,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_LANCTONF",        Param::PARAM_INPUT, $nrLanctonf,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDGRUPOFISC",     Param::PARAM_INPUT, $cdGrupoFisc,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDGRPFISCSV",     Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_IDINCINTEREST",   Param::PARAM_INPUT, $interEstad,    Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param("P_IDSISTEMA",       Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDFILIRECE",      Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDFORNECED",      Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDCLIENTE",       Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param("P_INIVALID",        Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDFILIEMIT",      Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_VRPRODUTO",       Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_LANCPRODUTO",     Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param("P_IDORIGEMPROD",    Param::PARAM_INPUT, $origemProd,    Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }
    
    protected function criaImpostos($cdFilial, $nrLanctonf, $cdTipoOper, $cdFiliOperLog, $dataEntrada, $vrBaseCalc, $seqItProd, $origemProd, $cdEmpresa, $cdArvProd, $cdInscEsta, $organizacao) {
       
        $interEstad  = 'N';
        
        $cdGrupoFisc = $this->callConsultaGrupofiscal('L', $cdFilial, $cdFilial, 'E', $cdTipoOper);
        
        $endeOrig    = $this->dfs48000SaidaDoOperadorLogisticoFactory->getEndeFili($cdFiliOperLog)[0];
     
        $endeDest    = $this->dfs48000SaidaDoOperadorLogisticoFactory->getEndeFili($cdFilial)[0];
        
        if($endeOrig["CDPAIS"] == $endeDest["CDPAIS"] && $endeOrig["SGESTADO"] != $endeDest["SGESTADO"]) {
            $interEstad = 'S';
        }
        

        
        $this->callConsultaIncidenciaImpostos('E', $cdEmpresa, $cdArvProd, $cdInscEsta, $cdTipoOper, $cdFilial, $nrLanctonf, $cdGrupoFisc, $interEstad, $origemProd);
        
        $incidencia = $this->dfs48000SaidaDoOperadorLogisticoFactory->getIncidencia($cdFilial.$nrLanctonf);
        
        foreach($incidencia as $inc) {
            $existe = false;
          
            $this->callConsultaAliquota($inc["CDIMPOTEMP"], 'E', $cdEmpresa, $cdArvProd, $cdInscEsta, $endeOrig["CDPAIS"], $endeOrig["SGESTADO"], $endeDest["CDPAIS"],
                $endeDest["SGESTADO"], ($inc["IDUTMUNIORIG"] == 'D') ? $endeDest["CDMUNICIPIO"]: $endeOrig["CDMUNICIPIO"], $vrBaseCalc, $dataEntrada, $cdFilial, $nrLanctonf, $cdGrupoFisc, $cdTipoOper, $origemProd);
       
            $aliquota = $this->dfs48000SaidaDoOperadorLogisticoFactory->getAliqTemp($cdFilial.$nrLanctonf);
            
            $taxaBc = (100 -  $aliquota["VRPREDBCTMP"])/100;
            $taxaAl = $aliquota["VRALIQUTMP"]  / 100;
            $baseCalc = $vrBaseCalc * $taxaAl;
            $imposto  = ($baseCalc * $taxaAl) - $taxaBc;
            $aliqOl   = $aliquota["VRALIQUTMP"];
            
            $seqItProdFormatado = str_pad($seqItProd, 4, "0", STR_PAD_LEFT);
             
            $this->dfs48000SaidaDoOperadorLogisticoFactory->insereTrabProImpos($cdFilial, $nrLanctonf, $seqItProdFormatado, $inc["CDIMPOTEMP"], $inc["CDFISCTEMP"], $aliquota["VRPREDBCTMP"],
                $aliquota["VRPARDEDTMP"], $baseCalc, $aliqOl, $imposto, $organizacao);
                
            foreach($this->prodImpos as &$prodImpo) {
                if($prodImpo["CDFILIIMPOIT"] == $cdFilial && $prodImpo["NRLANCIMPOIT"] == $nrLanctonf && $prodImpo["CDIMPOSTOIT"] == $inc["CDIMPOTEMP"]) {
                    $prodImpo["VRIMPOSIT"]      += $imposto;
                    $prodImpo["VRBCALCIMPOIT"]  += $baseCalc;
                    $prodImpo["VRCONTIMPOIT"]   += $baseCalc;
                    $this->dfs48000SaidaDoOperadorLogisticoFactory->updateTrabImpoNfTot($cdFilial, $nrLanctonf, $inc["CDIMPOTEMP"], $prodImpo["VRIMPOSIT"], '00000', '00000', $prodImpo["VRBCALCIMPOIT"], $organizacao);
                    $existe = true;
                }    
            }
            if(!$existe) {
                $this->dfs48000SaidaDoOperadorLogisticoFactory->insereTrabImpoNfTot($cdFilial, $nrLanctonf, $inc["CDIMPOTEMP"], $aliqOl, '00000', '00000', $baseCalc, $organizacao);
                $this->prodImpos[] = array("CDFILIIMPOIT" => $cdFilial, "NRLANCIMPOIT" => $nrLanctonf, "NRSEQIMPOIT" => $seqItProdFormatado, "CDIMPOSTOIT" => $inc["CDIMPOTEMP"], "CDFISCIMPOIT" => $inc["CDFISCTEMP"], 
                        "VRREDBCIMPOIT" => $aliquota["VRPREDBCTMP"], "VRPDEDIMPOIT" => $aliquota["VRPARDEDTMP"], "VRCONTIMPOIT" => $baseCalc, "VRBCALCIMPOIT" => $baseCalc, "VRIMPOSIT" => $imposto);
            }
         
            $this->dfs48000SaidaDoOperadorLogisticoFactory->deleteAliqTemp($cdFilial.$nrLanctonf);
            $this->dfs48000SaidaDoOperadorLogisticoFactory->deleteInicTemp($cdFilial.$nrLanctonf);
            
        }
    }
    
     public function callConsultaAliquota($cdImposto, $entrSaidAl, $cdEmpresa, $cdProduto, $cdInscEsta, $cdPaisOrig, $cdEstadoOrig, $cdPaisDest, 
        $cdEstadoDest, $cdMuniOrig, $vrBaseCalc, $iniInvalid, $cdFilial, $nrLanctonf, $cdGrupoFisc, $cdTipoOper, $origemProd) {
         
        $procedure = new StoredProcedure($this->connection, self::PROCEDURE_CONSULTA_ALIQUOTA);
 
        $procedure->addParam(new Param("P_IMPOSTO",           Param::PARAM_INPUT, $cdImposto,       Param::PARAM_TYPE_STR));   
        $procedure->addParam(new Param("P_ENTRSAIDAL",        Param::PARAM_INPUT, $entrSaidAl,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_EMPRESA",           Param::PARAM_INPUT, $cdEmpresa,       Param::PARAM_TYPE_STR));   
        $procedure->addParam(new Param("P_PRODUTO",           Param::PARAM_INPUT, $cdProduto,       Param::PARAM_TYPE_STR));   
        $procedure->addParam(new Param("P_INSCESTA",          Param::PARAM_INPUT, $cdInscEsta,      Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_PAISORIG",          Param::PARAM_INPUT, $cdPaisOrig,      Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_ESTAORIG",          Param::PARAM_INPUT, $cdEstadoOrig,    Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_PAISDEST",          Param::PARAM_INPUT, $cdPaisDest,      Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_ESTADEST",          Param::PARAM_INPUT, $cdEstadoDest,    Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_MUNIORIG",          Param::PARAM_INPUT, $cdMuniOrig,      Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_BASECALC",          Param::PARAM_INPUT, $vrBaseCalc,      Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_INIVALID",          Param::PARAM_INPUT, $iniInvalid,      Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_FILIAL",            Param::PARAM_INPUT, $cdFilial,        Param::PARAM_TYPE_STR));    
        $procedure->addParam(new Param("P_LANCTONF",          Param::PARAM_INPUT, $nrLanctonf,      Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_CDGRUPOFISC",       Param::PARAM_INPUT, $cdGrupoFisc,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDTIPOOPER",        Param::PARAM_INPUT, $cdTipoOper,      Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param("P_IDSISTEMA",           Param::PARAM_INPUT, null,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_LANCPRODUTO",         Param::PARAM_INPUT, null,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_VRPRODUTO",           Param::PARAM_INPUT, null,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDFILIRECE",          Param::PARAM_INPUT, null,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDFORNECED",          Param::PARAM_INPUT, null,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDCLIENTE",           Param::PARAM_INPUT, null,             Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param("P_IDORIGEMPROD",      Param::PARAM_INPUT, $origemProd,      Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }
    
    
    /******************************* GERA OCORRENCIA DE ATENDIMENTO *******************************
    *                                           GQL                                               *
    ***********************************************************************************************/
    protected function geraOcorrenciaDeAtendimento($produto, $notaFiscal, $dadosDevolucao, $proxLancamento, $cdOperador, $nmOperador, $cdFilDest, $cdFilEmit, $organizacao) {
        $motivoDev = $this->dfs48000SaidaDoOperadorLogisticoFactory->getMotivoDev($produto["CDMOTIVODEV"]);
        $motivoDev["IDGERAOCORATE"] = 'S';
        
        if($motivoDev["IDGERAOCORATE"] == 'S') {
            $nrMovOcorr  = $this->novoCodigo->getCodeByTableName(self::NOVO_CODIGO_MOVOCORR.$cdFilDest, $organizacao);
            
            $cdFornOpLog = $this->dfs48000SaidaDoOperadorLogisticoFactory->getCdFornOpLog($cdFilEmit)["CDFORNOPELOG"];
        
            $notaOrigem  = $this->dfs48000SaidaDoOperadorLogisticoFactory->getNotaOrigem($notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"]);
            
            $this->dfs48000SaidaDoOperadorLogisticoFactory->insereLote($notaOrigem["CDFILIRECE"], $produto["NRLOTEESTQ"], $produto["NRSUBLOTE"], $produto["CDPRODUTO"]);
            
            $this->dfs48000SaidaDoOperadorLogisticoFactory->insereMovOcorr($cdFilDest, $nrMovOcorr, $notaOrigem["CDFILIRECE"], $cdFornOpLog, $dadosDevolucao["DTSAIDA"], $nmOperador,
                $cdOperador, $produto["CDPRODUTO"], $produto["NRLOTEESTQ"], $produto["NRSUBLOTE"], '', $notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"], $produto["CDMARCA"],
                $dadosDevolucao["DTSAIDA"], 'P', $organizacao);
            
            $dsMovOcorr = $this->strToHex($motivoDev["NMMOTIVODEV"]);
            
            $this->dfs48000SaidaDoOperadorLogisticoFactory->insereDsMovOcorr($cdFilDest, $nrMovOcorr, $dsMovOcorr, $organizacao);
        }
        
        if($motivoDev["IDGERAPEDOL"] == 'S') {
            $nfRelacPexOl = $this->dfs48000SaidaDoOperadorLogisticoFactory->getRelacPexOl($notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"]);
            
            
            if(isset($nfRelacPexOl[0])) {
                $nfRelacPexOl = $nfRelacPexOl[0];
                
                $dtPedPexOl = date('d-m-Y', strtotime("+1 days", strtotime($this->formataDataParaComparacao($motivoDev["DATADIA"]))));
                
                
                $nrPedPexOl = $this->novoCodigo->getCodeByTableName(self::NOVO_CODIGO_NRPEXOL.$nfRelacPexOl["CDFILDEST"].$nfRelacPexOl["CDFILLANC"], $organizacao);
                $this->dfs48000SaidaDoOperadorLogisticoFactory->inserePedExol($nfRelacPexOl["CDFILDEST"], $nfRelacPexOl["CDFILLANC"], $nrPedPexOl, $dtPedPexOl, '1', $nfRelacPexOl["CDEMPRESA"],
                                                                        $nfRelacPexOl["CDFILIOPELOG"], $nfRelacPexOl["IDTIPOEXOL"], $nfRelacPexOl["IDPEDEXOLEXTR"], $notaFiscal["CDFILIAL"].$notaFiscal["NRLANCTONF"], $organizacao);
            
                $lotePedido = '';
                // Lote pedido
                if(empty($motivoDev["CDLOTEENVPEDPRI"])) {
                    $lotePedido = $motivoDev["CDLOTEENVPEDSEG"];
                }
                else if(empty($motivoDev["CDLOTEENVPEDSEG"])) {
                    $lotePedido = $motivoDev["CDLOTEENVPEDPRI"];
                }
                else if(!(empty($motivoDev["CDLOTEENVPEDPRI"])) && !(empty($motivoDev["CDLOTEENVPEDSEG"]))) {
                    $primeiraHora = $this->formataDtHora($motivoDev["DSHORALIMPRI"]);
                    $segundaHora  = $this->formataDtHora($motivoDev["DSHORALIMSEG"]);
                    $horaBanco    = $this->formataDtHora($motivoDev["HORABANCO"]);
                    
                    if(!empty($primeiraHora) && !empty($segundaHora)) {
                        if((strtotime($horaBanco) < strtotime($primeiraHora)) && (strtotime($horaBanco) < strtotime($segundaHora))) {
                            if(strtotime($primeiraHora) < strtotime($segundaHora)) {
                                $lotePedido = $motivoDev["CDLOTEENVPEDPRI"];
                            }
                            else {
                                $lotePedido = $motivoDev["CDLOTEENVPEDSEG"];
                            }
                        }
                        else if((strtotime($horaBanco) < strtotime($primeiraHora)) && (strtotime($horaBanco) > strtotime($segundaHora))) {
                            $lotePedido = $motivoDev["CDLOTEENVPEDPRI"];
                        }
                        else if((strtotime($horaBanco) > strtotime($primeiraHora)) && (strtotime($horaBanco) > strtotime($segundaHora))) {
                             if(strtotime($primeiraHora) < strtotime($segundaHora)) {
                                $lotePedido = $motivoDev["CDLOTEENVPEDPRI"];
                            }
                            else {
                                $lotePedido = $motivoDev["CDLOTEENVPEDSEG"];
                            }
                        }
                    }
                }
            }
            
            $datePedExOl = date('d/m/Y h:i:s', strtotime("+1 days", strtotime(str_replace("/", "-", $motivoDev["DATADIA"]))));
            
            $this->dfs48000SaidaDoOperadorLogisticoFactory->insereItemPedExOl($nfRelacPexOl["CDFILDEST"], $nfRelacPexOl["CDFILLANC"], $nrPedPexOl, $produto["CDPRODUTO"], $produto["QTDEVOLUCAOOL"], 
                                                                    $datePedExOl, $produto["CDMARCA"], $lotePedido, $organizacao);
        }
    }
    
    
    /************************************ ATUALIZA STATUS AF *************************************/
    protected function atualizaStatusAF($produto, $notaFiscal, $dadosDevolucao, $cdFilDest) {
        $itemAutAf    = $this->dfs48000SaidaDoOperadorLogisticoFactory->getItemAutAf($cdFilDest, $notaFiscal["NRLANCTONF"], $produto["CDMARCA"], $produto["CDPRODUTO"])[0];
        
        if($produto["QTDEVOLUCAOOL"] == $itemAutAf["QTEFETENTRAF"]) {
            $idBaixitmAF = '';
            $qtEfetentAF = 0;
        }
        else {
            $idBaixitmAF = '1';
            $qtEfetentAF = $produto["QTITEMPROD"] - $produto["QTDEVOLUCAOOL"];
        }
        
        $this->dfs48000SaidaDoOperadorLogisticoFactory->updateItemAutAf($qtEfetentAF, $idBaixitmAF, $itemAutAf["CDFILDEST"], $itemAutAf["CDFILLANC"], $itemAutAf["NRAUTOFORN"],
                                                                    $itemAutAf["CDMARCA"], $itemAutAf["CDPRODUTO"], $itemAutAf["DTPROGENTAF"]);
                                                                    
        $itemAutAf1 = $this->dfs48000SaidaDoOperadorLogisticoFactory->getItemAutAf2($itemAutAf["CDFILDEST"], $itemAutAf["CDFILLANC"], $itemAutAf["NRAUTOFORN"]);
        if(isset($itemAutAf1[0])) {
            if( $itemAutAf1[0]["NRAUTOFORN"] == "" || $itemAutAf1[0]["NRAUTOFORN"] == "0" ) $this->dfs48000SaidaDoOperadorLogisticoFactory->updateAutoForne('', $itemAutAf["CDFILDEST"], $itemAutAf["CDFILLANC"], $itemAutAf["NRAUTOFORN"]);
            else $this->dfs48000SaidaDoOperadorLogisticoFactory->updateAutoForne('1', $itemAutAf["CDFILDEST"], $itemAutAf["CDFILLANC"], $itemAutAf["NRAUTOFORN"]);
        }
        else {
            $this->dfs48000SaidaDoOperadorLogisticoFactory->updateAutoForne('', $itemAutAf["CDFILDEST"], $itemAutAf["CDFILLANC"], $itemAutAf["NRAUTOFORN"]);
        }
    }
    
    /********************************** INSERE DADOS ADICIONAIS **********************************/
    protected function insereDadosAdicionais($notaFiscal, $proxLancamento, $cdFilDest, $cdFilEmit, $dadosDevolucao, $organizacao, $paramsGeracao) {
        $nrNotaFisc     = $notaFiscal["NRLANCTONF"];
        $dsDadosAdic    = "Filial Origem: $cdFilDest  / Filial Destino:  $cdFilEmit / NF Origem: $nrNotaFisc";
        
        if($notaFiscal['USR_NMROTATRANS'] != NULL) {
            $adqMaxDadosAdic = $this->adqMaxDadosAdic($dadosDevolucao['CDFILIOPELOG'], $notaFiscal['NRLANCTONF']);
            
            if(!empty($adqMaxDadosAdic)) {
                $this->dfs48000SaidaDoOperadorLogisticoFactory->insereDadosAdic($notaFiscal["CDFILDEST"], $proxLancamento["P_PRXLANCTONF"], $adqMaxDadosAdic[0]['NRDADOADIC'], '', $dsDadosAdic, $organizacao);
                
                if($paramsGeracao['IDSISDOS'] == '3') {
                    $adqDadosAdicAF = self::adqDadosAdicAF($dadosDevolucao['CDFILIOPELOG'], $notaFiscal['NRLANCTONF']);
                    
                    if(!empty($adqDadosAdicAF)) {
                        $adqMaxDadosAdic = $this->adqMaxDadosAdic($dadosDevolucao['CDFILIOPELOG'], $notaFiscal['NRLANCTONF']);
                        
                        $dsDadosAdic = "UnOp Emitente: " . $adqDadosAdicAF[0]['CDFILIEMIT'] . " - " . $adqDadosAdicAF[0]['NMFILIEMIT'];
                        
                        $this->dfs48000SaidaDoOperadorLogisticoFactory->insereDadosAdic($notaFiscal["CDFILDEST"], $proxLancamento["P_PRXLANCTONF"], $adqMaxDadosAdic[0]['NRDADOADIC'], '', $dsDadosAdic, $organizacao);
                        
                        $adqMaxDadosAdic = $this->adqMaxDadosAdic($dadosDevolucao['CDFILIOPELOG'], $notaFiscal['NRLANCTONF']);
                        
                        $dsDadosAdic = "UnOp Centralizadora: " . $adqDadosAdicAF[0]['CDFILIFISC'] . " - " . $adqDadosAdicAF[0]['NMFILIFISC'];
                        
                        $this->dfs48000SaidaDoOperadorLogisticoFactory->insereDadosAdic($notaFiscal["CDFILDEST"], $proxLancamento["P_PRXLANCTONF"], $adqMaxDadosAdic[0]['NRDADOADIC'], '', $dsDadosAdic, $organizacao);
                        
                        $adqMaxDadosAdic = $this->adqMaxDadosAdic($dadosDevolucao['CDFILIOPELOG'], $notaFiscal['NRLANCTONF']);
                        
                        $dsDadosAdic = "UnOp Destino: " . $adqDadosAdicAF[0]['CDFILIRECE'] . " - " . $adqDadosAdicAF[0]['NMFILIRECE'];
                        
                        $this->dfs48000SaidaDoOperadorLogisticoFactory->insereDadosAdic($notaFiscal["CDFILDEST"], $proxLancamento["P_PRXLANCTONF"], $adqMaxDadosAdic[0]['NRDADOADIC'], '', $dsDadosAdic, $organizacao);
                        
                        $adqMaxDadosAdic = $this->adqMaxDadosAdic($dadosDevolucao['CDFILIOPELOG'], $notaFiscal['NRLANCTONF']);
                        
                        $dsDadosAdic = "Local de Entrega: " . $adqDadosAdicAF[0]['DSENDEFILISEP'] . ", " . $adqDadosAdicAF[0]['NRENDEFILISEP'] . " - " . $adqDadosAdicAF[0]['DSCOMPENDFIL'] . " - " . $adqDadosAdicAF[0]['NMBAIRFILI'] . " - " . $adqDadosAdicAF[0]['NMMUNICIPIO'] . " - " . $adqDadosAdicAF[0]['SGESTADO'] . " - " . $adqDadosAdicAF[0]['NMPAIS'] . " - " . $adqDadosAdicAF[0]['NRCEPFILI'];
                        
                        $this->dfs48000SaidaDoOperadorLogisticoFactory->insereDadosAdic($notaFiscal["CDFILDEST"], $proxLancamento["P_PRXLANCTONF"], $adqMaxDadosAdic[0]['NRDADOADIC'], '', $dsDadosAdic, $organizacao);
                    }
                }
            }
        }
    }
    
    /******************************* VERIFICA IMPOSTOS OBRIGATORIOS *******************************
    *             Verifica se a nota e seus produtos possuem os impostos obrigatorios             *
    ***********************************************************************************************/
    protected function verififcaImpostosObrigatorio($cdFilial, $nrLanctonf) {
        $notaFiscal   = $this->dfs48000SaidaDoOperadorLogisticoFactory->getDadosNf($cdFilial, $nrLanctonf);
        $impostosObri = $this->dfs48000SaidaDoOperadorLogisticoFactory->getImpostoObrigatorio($notaFiscal["IDENTRSAIDOP"], $notaFiscal["CDTIPOOPER"]);
        $impostosErro = '';
        $produtosComErro = false;
        $notaErro        = false;
        
        foreach ($impostosObri as $impObri) {
            if($impObri["IDPRODNOTA"] == 'P') {
                $impostoProdutos = $this->dfs48000SaidaDoOperadorLogisticoFactory->getImpostoProdutos($cdFilial, $nrLanctonf, $impObri["CDIMPOSTO"]);

                foreach ($impostoProdutos as $impostoProduto) {
                    $produtosComErro = true;
                    array_push($this->arrayImpostosErro, array("NMPRODUTO"  => $impostoProduto["NMPRODUTO"],
                                                         "DSERRO"     => "Produto sem incidência de imposto obrigatório. Imposto: ".$impObri["SGIMPOSTO"]));
                }
            }
            else {
                $impostosCorpo   = $this->dfs48000SaidaDoOperadorLogisticoFactory->getImpostoNota($cdFilial, $nrLanctonf, $impObri["CDIMPOSTO"]);
                if(!isset($impostosCorpo[0])) {
                    $notaErro = true;   
                    $impostosErro.= $impObri["SGIMPOSTO"].', ';
                }
                else if($impostosCorpo[0]["CDFILIAL"] == null) {
                    $notaErro = true;   
                    $impostosErro.= $impObri["SGIMPOSTO"].', ';
                }
            }
        }
        
        if($produtosComErro) throw new \Exception("Notafiscal possui produtos sem incidência de imposto(s) obrigatório(s).");
        $size = strlen($impostosErro);

        if($size != 0) {
            $impostosErro = substr($impostosErro,0, $size-2);
            throw new \Exception("Nota Fiscal sem incidência do(s) imposto(s) obrigatório(s) ".$impostosErro.'.');
        }
        
    }
    
    /******************************* VERIFICA IMPOSTOS OBRIGATORIOS *******************************
    *                                   Gera Rateio Para a nota                                   *
    ***********************************************************************************************/
    protected function geraRateio($cdFilial, $nrLanctonf, $idEntrSaid, $rateioNf, $organizacao, $autMan = 'N', $realizaManut = 'P') {
        if($realizaManut != 'S') {
            if($realizaManut == 'P') {
                $this->dfs48000SaidaDoOperadorLogisticoFactory->deleteRateioNf($cdFilial, $nrLanctonf);
                $notaFiscal = $this->dfs48000SaidaDoOperadorLogisticoFactory->getNotaFiscalRateio($cdFilial, $nrLanctonf);
                //$consContaRateio = $this->dfs48000SaidaDoOperadorLogisticoFactory->getConsContaRateio($idEntrSaid, $notaFiscal["CDTIPOOPER"]);
                
                if($idEntrSaid == 'S') {
                    
                    $tpContaReceber = $this->dfs48000SaidaDoOperadorLogisticoFactory->getTipoContaReceber($cdFilial, $nrLanctonf);
                    $produtosRateio = $this->dfs48000SaidaDoOperadorLogisticoFactory->getItemProdutoRateio($cdFilial, $nrLanctonf);
                    
                    foreach ($produtosRateio as $produto) {
                        $ratCtpr = $this->dfs48000SaidaDoOperadorLogisticoFactory->getRatTpCtpr(isset($tpContaReceber[0]) ? $tpContaReceber[0]["CDTPCTRECEB"] : '');
                        
                        foreach ($ratCtpr as $rat) {
                            $filiRateio = ($rat["CDFILIAL"]) ? $rat["CDFILIAL"] : $notaFiscal["CDFILIEMIT"];
                            $rateioExistente = $this->dfs48000SaidaDoOperadorLogisticoFactory->getRateioNf($cdFilial, $nrLanctonf, $produto["NRSEQITPROD"], $filiRateio, $rat["CDCENTCUST"]);
                            $vrRateNf   = $produto["VRCONTITEM"] * ($produto["VRPERATTPCTR"] / 100);
                            if(isset($rateioExistente[0])) {
                                $this->dfs48000SaidaDoOperadorLogisticoFactory->updateRateioNf($vrRateNf, $rat["VRPERATTPCTR"], $cdFilial, $nrLanctonf, $produto["NRSEQITPROD"], $filiRateio, $rat["CDCENTCUST"]);
                            }
                            else $this->dfs48000SaidaDoOperadorLogisticoFactory->insereRateioNf($vrRateNf, $rat["VRPERATTPCTR"], $cdFilial, $nrLanctonf, $produto["NRSEQITPROD"], $filiRateio, $rat["CDCENTCUST"], $organizacao);
                        }
                    }
                }
                else {
                    $this->insereRateioNf2($cdFilial, $nrLanctonf, $organizacao);
                }
            }
            else {
                
                // Não utilizado por esta rotina
            }
        }
    }
    
    /********************************** VALIDACOES DA NOTAFISCAL **********************************
    * Faz as validações necessarias para a nota                                                   *
    * Caso a nota não seja aprovada em alguma das validações uma exceção é retornada              *
    * e a nota é adicionada ao array de notas com erro $arrayNotasErro                            *
    ***********************************************************************************************/
    protected function validaNotaSaidaOl($notaFiscal, $paramsGeracao, $dtSaida, $cdSerie, $tipoOperEnt) {
        $this->validacoesParamCtrlFisOl($dtSaida, $paramsGeracao, $paramsGeracao["DTPROCESSA"], $paramsGeracao["NRDIASSAIDFIL"], $paramsGeracao["NRDIASSAID"], $paramsGeracao["NRDIASSAIDPD"], $paramsGeracao["NRDIASSAIDPDFIL"]);  
        
        $this->validaSerie($cdSerie);
        $this->validaDataIntegracao($paramsGeracao["CDEMPRESA"], 'S', $tipoOperEnt, $dtSaida);
        $this->validaValorNota($notaFiscal["VRNOTAFISC"]);
        $this->validaDataEmissao($notaFiscal["DTEMISSAO"], $dtSaida);
        $this->validaNrNotaFisc($notaFiscal["NRNOTAFISC"]);
    }
    
    private function validacoesParamCtrlFisOl($dtSaida, $paramsGeracao, $dataProcessamento, $nrDiasSaidFil, $nrDiasSaid, $nrDiasSaidPd, $nrDiasSaidPdFil) {
        $dataProcessamento = $this->formataDataParaComparacao($dataProcessamento);
        $dtSaida = $this->formataDataParaComparacao($dtSaida);
        
        if($paramsGeracao["IDLANCDTMAI"] === 'N' && $paramsGeracao["IDLANCDTMAIFIL"] === 'N' && (strtotime($dtSaida) > strtotime($dataProcessamento)))
            throw new \Exception(self::MSG_ERROS["DAT_MAI"]);
         
        if($paramsGeracao["IDLANCDTMEN"] === 'N' && $paramsGeracao["IDLANCDTMENFIL"] === 'N' && (strtotime($dtSaida) < strtotime($dataProcessamento))) 
            throw new \Exception(self::MSG_ERROS["DAT_MEN"]); 
            
        if($paramsGeracao["IDLANCDTMENFIL"] === 'S' && (strtotime($dtSaida) < strtotime($dataProcessamento))) {
            if(strtotime($dtSaida) < strtotime("-$nrDiasSaidFil days", strtotime($dataProcessamento)))   {
                throw new \Exception(self::MSG_ERROS["DAT_M"]); 
            }
        } else {
            if($paramsGeracao["IDLANCDTMEN"] === 'S' && (strtotime($dtSaida) < strtotime($dataProcessamento))) 
                if(strtotime($dtSaida) < strtotime("-$nrDiasSaid days", strtotime($dataProcessamento)))
                    throw new \Exception(self::MSG_ERROS["DAT_M"]);
        }
        
        if($paramsGeracao["IDLANCDTMAIFIL"] === 'S' && (strtotime($dtSaida) > strtotime($dataProcessamento))) {
            if(strtotime($dtSaida) > strtotime("+$nrDiasSaidPdFil days", strtotime($dataProcessamento))) {
                throw new \Exception(self::MSG_ERROS["DAT_M"]);
            }
        } else {
            if($paramsGeracao["IDLANCDTMAI"] === 'S' && (strtotime($dtSaida) > strtotime($dataProcessamento)))  {
                if(strtotime($dtSaida) > strtotime("+$nrDiasSaidPd days", strtotime($dataProcessamento)))
                    throw new \Exception(self::MSG_ERROS["DAT_M"]);
            }
        }
    }
    
    private function validaSerie($cdSerie) {
        if(!$cdSerie) throw new \Exception(self::MSG_ERROS["SERIE_PAD"]);
    }
    
    private function validaDataIntegracao($cdEmpresa, $idEntrSaidOp, $cdTipoOperac, $dtReferencia) {
        $contopctbl = $this->dfs48000SaidaDoOperadorLogisticoFactory->getContopectbl($cdEmpresa);
        
        if($contopctbl) {
            if($contopctbl[0]) {
                if($idEntrSaidOp == 'E') {
                    if (strtotime($this->formataDataParaComparacao($dtReferencia)) >= $this->getDataTime($contopctbl[0]["DTINIINTCONTDFE"]) && strtotime($this->formataDataParaComparacao($dtReferencia)) <= $this->getDataTime($contopctbl[0]["DTFININTCONTDFE"]))
                        return true;
                    else throw new \Exception(self::MSG_ERROS["LANC_FORTADT"]);
                } else if($idEntrSaidOp == 'S') {
                    if (strtotime($this->formataDataParaComparacao($dtReferencia)) >= strtotime($this->formataDataParaComparacao($contopctbl[0]["DTINIINTCONTDFS"])) && strtotime($this->formataDataParaComparacao($dtReferencia)) <= strtotime($this->formataDataParaComparacao($contopctbl[0]["DTFININTCONTDFS"]))) 
                        return true;
                    else throw new \Exception(self::MSG_ERROS["LANC_FORTADT"]);
                }
            } else return true;
        } else return true;
    }
    
    private function validaValorNota($vrNotaFisc) {
        if(!($vrNotaFisc > 0)) throw new \Exception(self::MSG_ERROS["TOTAL_NF_ZERADO"]);
    }
    
    private function validaDataEmissao($dtEmissao, $dtEntrada) {
        if(is_null($dtEmissao)) throw new \Exception(self::MSG_ERROS["DTEMISSAO_VAZIA"]);
    }
    
    private function validaNrNotaFisc($nrNotaFisc) {
        if(is_null($nrNotaFisc)) throw new \Exception(self::MSG_ERROS["NRNOTAFISC_VAZIO"]);
    }
    
    /**************************** VALIDACOES DOS PRODUTOS DA NOTAFISCAL ****************************
    * Faz as validações necessarias para o produto                                                 *
    * Caso o produto não seja aprovada em alguma das validações uma exceção é retornada            *
    * e o produto é adicionada ao array de produtos com erro $arrayNotasErro                       *
    ************************************************************************************************/
    // protected function validaProduto($produto, $notaFiscal, $posicaoEstoqueProd) {
    //      $this->validaProdutoInativo($produto, $notaFiscal["NRLANCTONF"], $notaFiscal["CDFILDEST"]);
    //     $this->validaValorProduto($produto["VRUNITITEM"]);
    //     // $this->validaQtDevolucao($produto["QTDEVOLUCAOOL"]);
    //      $this->validaQuantidadeProd($notaFiscal["CDFILDEST"], $notaFiscal["NRLANCTONF"], $produto["CDPRODUTO"], $produto["CDMARCA"], $produto);
        
    //     // $this->validaEstoqueDiaProduto($produto, $posicaoEstoqueProd);
        
    // }
    protected function validaProduto($produto, $notaFiscal, $cdFilial, $sugRegAux, $cdFilDest) {
        $this->validaProdutoPorFilial($cdFilDest, $produto["CDPRODUTO"]);
        $this->validaValorProduto($produto["VRUNITITEM"]);
        $this->validaQuantidadeProd($cdFilial, $notaFiscal["NRLANCTONF"], $produto["CDPRODUTO"], $produto["CDMARCA"], $produto);
    }
    
    private function validaProdutoInativo($produto, $nrLanctonf, $cdFilDest) {
        $prodFili = $this->dfs48000SaidaDoOperadorLogisticoFactory->getProdFili($cdFilDest, $produto["CDPRODUTO"]);
       
        if($prodFili["IDPRODATIV"] !== 'S') throw new \Exception(self::MSG_ERROS["PRODUTO_INATIVO"]);
        if(isset($prodFili[0])) {
            if($prodFili[0]["IDPRODATIV"] == 'I') throw new \Exception(self::MSG_ERROS["PRODUTO_INATFIL"]);
        }
        else throw new \Exception(self::MSG_ERROS["PRODUTO_FILIAL"]);
    }
    
    private function validaValorProduto($valorProduto) {
        if(!($valorProduto > 0)) throw new \Exception(self::MSG_ERROS["PRECO_ZERADO"]);
    }
    
    private function validaQtDevolucao($qtDevolucao) {
        if($qtDevolucao == 0) throw new \Exception(self::MSG_ERROS["QTDE_ZERADA"]);
    }
    
    private function validaEstoqueDiaProduto($produto, $posicaoEstoqueProd) {
        if($posicaoEstoqueProd["QTESTOQDIA"] < ($produto["QTDEVOLUCAOOL"] * $produto["VRFATOCONV"]) 
            && $produto["IDCNTRESTOQ"] != 'N') {
                throw new \Exception(self::MSG_ERROS["EST_INSUFICIENTE"]);
        }
    }
    
    /***************************************** PROCEDURES *****************************************/
    
    protected function atualizaEstoque($cdFilial, $numLancamento, $dtInicial, $dtFinal, $nrLancto = '') {
        $procedure = new StoredProcedure($this->connection, self::PROCEDURE_ATUALIZA_ESTOQUE);
        $procedure->addParam(new Param('P_FILIMOVI',    Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILILANCTO',  Param::PARAM_INPUT, $numLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINICIAL',   Param::PARAM_INPUT, $dtProcessamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTFINAL',     Param::PARAM_INPUT, $numLancEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTO',    Param::PARAM_INPUT, $nrLancto, Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    
    }
    
    
    /************************************* FUNÇÕES AUXILIARES *************************************/
    
    public function operacaoAtualizaEstoque($cdTipoOper, $idEntrSaidOp = 'S') {
        $infoOperac = $this->dfs48000SaidaDoOperadorLogisticoFactory->getInfoTipoOperac($cdTipoOper, $idEntrSaidOp)[0];
        return !!($infoOperac["IDATUESTTPOP"] == 'S');
    }
    
    protected function getDataTime($data) {
        $data = $this->formataDataParaComparacao($data);
        return strtotime($data);
    }
    
    protected function formataDataParaComparacao($data) {
        $data = substr($data, 0, 10);
        return str_replace("/", "-", $data);
    }
    
    private function maskProduto($cdarvprod, $nrorg) {
        return $this->masks->mask($cdarvprod, preg_replace("/([^.])/", '#', PictureProduto::mascaraProduto($nrorg)));
    }
    
    public function strToHex($string){
        $hex='';
        for ($i=0; $i < strlen($string); $i++){
            $hex .= dechex(ord($string[$i]));
        }
        return $hex;
    }
    
    public function hexToStr($hex){
        $string='';
        for ($i=0; $i < strlen($hex)-1; $i+=2){
            $string .= chr(hexdec($hex[$i].$hex[$i+1]));
        }
        return $string;
    }
    
    public function formataDtHora($dtHora) {
        if(!empty($dtHora)) {
            $dtHora = substr($dtHora, 0, 2).':'.substr($dtHora, 2, 3);
            $date = date('H:i', strtotime($dtHora));
            return $date;
        }
        return null;
    }
    
    /************************************* FUNÇÕES PARA ERROS **************************************     
    * Funções que retornam os erros durante a execução da devolução                                *
    ************************************************************************************************/

    public function getErros() {
        return array_merge($this->getArrayNotasErro(), $this->getArrayProdutosErro());
    }
    
    public function getQtNotasSucesso() {
        return $this->notasSucesso;
    }
    
    public function getArrayNotasErro() {
        return $this->arrayNotasErro;
    } 
    
    public function getArrayProdutosErro() {
        return $this->arrayProdutosErro;
    }
    
    public function getArrayImpostosErro() {
        return $this->arrayImpostosErro;
    }
    
    public function getDtProcessaOpl($row) {
        try {
            $params = array(
                "P_FILOPELOG"  => $row['CDFILIOPELOG'],
                "P_DTENTRSAID" => $row['DTENTRSAID'],
            );
            $query = "SELECT QUERY_DTENTRSAID_DFS48000(:P_FILOPELOG, :P_DTENTRSAID) MESSAGE FROM DUAL";
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function validaTipoOp($row) {
        try {
            $params = array(
                "P_CDFILIOPELOG"  => $row['CDFILIOPELOG'],
                "P_CDFILIAL" => $row['CDFILIAL'],
            );
            $query = "SELECT TIPOOPEROPL.CDTIPOOPERSAID, TIPOOPEROPL.CDTIPOOPERENTR FROM TIPOOPEROPL WHERE TIPOOPEROPL.CDFILIOPELOG  =  :P_CDFILIOPELOG AND INSTR(:P_CDFILIAL, TIPOOPEROPL.CDFILDEST) > 0 AND TIPOOPEROPL.CDTIPOOPERSAID IS NOT NULL AND TIPOOPEROPL.CDTIPOOPERENTR IS NOT NULL";
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function getDtProcessaAtual($row) {
        try {
            $params = array(
                "P_CDFILIALOL"  => $row['CDFILIOPELOG']
            );
            $query = "SELECT TO_CHAR(CASE
                                          WHEN PARAMGERAL.IDUTILDTPROC = 'S'
                                          THEN NVL(PARAMFILIAL.DTPROCESSA, TRUNC(SYSDATE))
                                          ELSE TRUNC(SYSDATE)
                                       END, 'DD/MM/YYYY') DTPROCESSA
                          FROM PARAMGERAL, PARAMFILIAL
                         WHERE PARAMFILIAL.CDFILIAL = :P_CDFILIALOL";
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function validaItensPedido($row, $dados) {
        try {
            $params = array(
                "P_CDFILIOPELOG"        => $dados['CDFILIOPELOG'],
                "P_CDFILDEST"           => $row['CDFILIAL'],
                "P_CDFILLANC"           => $row['CDFILLANC'],
                "P_NRAUTOFORN"          => $row['NRAUTOFORN'],
                "P_SUBGRUPO"            => $dados['SUBGRUPO'],
                "P_DATAENT"             => $dados['DATAENT'],
                "P_DTENTREGA"           => $dados['DTENTREGA'],
                "P_DTPROC"              => $dados['DTPROC'],
                "P_PEDOLEXTR"           => $dados['PEDOLEXTR'],
                "P_CDLOTENVIOL"         => $dados['CDLOTENVITOL'],
                "P_IDBAIXAOLSAI"        => "W",
                "P_IDUTILCONSSOLIOL"    => $dados['IDUTILCONSSOLIOL'],
                "P_IDUTRESPEDXOL"       => $dados['IDUTRESPEDXOL'],
                "P_DTSAIDANF"           => $dados['DATA'],
                "P_GERA_NF"             => $dados['GERANF'],
                "P_WEBSERVICE"          => $dados['WEBSERVICE']
            );
            $query = "SELECT IDCHECKPRAF, CDFILIAL, CDARVPROD, NMPRODUTO, CDPRODUTO,
                        SGUNIDADE, CDMARCA, CDMARCAOL, CDMARCAAF, NMMARCA, NRAUTOFORN, CDFILLANC,
                        NRLANCTONF, IDBAIXITAFS, CDFILDEST, IDUTILITEM, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ,
                        NRSUBLOTE, IDUTQTESTRANG, CDCLIENTEEXT, CDARVPRODOL, NMPRODUTOOL, CDARVPRODEST, NMPRODUTOEST,
                        CDFILIOPELOG, NMFILIOPELOG, NMFILIAL, LISTAERRO, DTPROGEMBARQ, DTPROGENTAF, DTPROGENPEOL, QTPROGENTAF,
                        QTSAIDOPLOG,LISTAERRO, VRPREUNIAF, QTNFRELAFSAI, QTATENDPEDOL, VRPBRUTOPEDOL, VRPLIQUIPEDOL, QTATENDESTRANG,
                        QTESTOQDIA, QTRESERVESTOL, ESTOQ_DISPO, RESERVA_ITEM, QTESTOQDIAEST, VRFATOCONVOL, VRFATOCONV, CDPRODUTOEST
                    FROM TABLE (QUERY_ITEM_DFS48000(:P_CDFILIOPELOG, :P_CDFILDEST, :P_CDFILLANC, :P_NRAUTOFORN, :P_SUBGRUPO,
                                                    :P_DATAENT, :P_DTENTREGA, :P_DTPROC, :P_PEDOLEXTR, :P_CDLOTENVIOL,
                                                    :P_IDBAIXAOLSAI, :P_IDUTILCONSSOLIOL, :P_IDUTRESPEDXOL, :P_DTSAIDANF, :P_GERA_NF,
                                                    :P_WEBSERVICE))";
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
              
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function getStWebService($row) {
        try {
            $params = array(
                "P_CDFILIOPELOG"  => $row['CDFILIOPELOG']
            );
            $query = "SELECT CDFILIAL
                       FROM PARAMFILIAL
                      WHERE CDFILIAL = :P_CDFILIOPELOG
                        AND CDFILWMSMHA  IS NOT NULL
                        AND CDOPERWMSMHA IS NOT NULL";
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqMaxDadosAdic($cdfilial, $nrLanctonf) {
        try {
            $params = array(
                "P_CDFILIAL"   => $cdfilial,
                "P_NRLANCTONF" => $nrLanctonf
            );
            $query = "SELECT NVL(MAX(NRDADOADIC),0) + 1  NRDADOADIC
                          FROM DADOSADIC
                         WHERE CDFILIAL   = :P_CDFILIAL
                           AND NRLANCTONF = :P_NRLANCTONF";
   
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqVerifDuplNF($cdfiliopelog, $cdfildest, $cdfillanc, $produto) {
        try {
            $params = array(
                "P_CDFILIAL"    => $cdfiliopelog,
                "P_CDFILDEST"   => $cdfildest,
                "P_CDFILLANC"   => $cdfillanc,
                "P_NRAUTOFORN"  => $produto['NRAUTOFORN'],
                "P_CDMARCA"     => $produto['CDMARCAOL'],
                "P_CDPRODUTO"   => $produto['CDPRODUTOEST'],
                "P_DTPROGENTAF" => substr($produto['DTPROGENTAF'], 0, 10)
            );
            $query = "SELECT NRLANCTONF
                         FROM NFRELACAFSAI
                        WHERE CDFILIAL     = :P_CDFILIAL
                          AND CDFILDEST    = :P_CDFILDEST
                          AND CDFILLANC    = :P_CDFILLANC
                          AND NRAUTOFORN   = :P_NRAUTOFORN
                          AND CDMARCA      = :P_CDMARCA
                          AND CDPRODUTO    = :P_CDPRODUTO
                          AND DTPROGENTAF  = :P_DTPROGENTAF
                       UNION
                       SELECT NRLANCTONF
                         FROM NFRELACPEXOL
                        WHERE CDFILIAL     = :P_CDFILIAL
                          AND CDFILDEST    = :P_CDFILDEST
                          AND CDFILLANC    = :P_CDFILLANC
                          AND NRPEDEXOL    = :P_NRAUTOFORN
                          AND CDMARCA      = :P_CDMARCA
                          AND CDPRODUTO    = :P_CDPRODUTO
                          AND DTPROGENPEOL = :P_DTPROGENTAF";
   
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqNFRelacAFSai($cdfiliopelog, $nrlanctonf, $cdfildest, $cdfillanc, $produto) {
        try {
            $params = array(
                "P_CDFILIAL"    => $cdfiliopelog,
                "P_NRLANCTONF"  => $nrlanctonf,
                "P_CDFILDEST"   => $cdfildest,
                "P_CDFILLANC"   => $cdfillanc,
                "P_NRAUTOFORN"  => $produto['NRAUTOFORN'],
                "P_CDMARCA"     => $produto['CDMARCAOL'],
                "P_CDPRODUTO"   => $produto['CDPRODUTO'],
                "P_DTPROGENTAF" => substr($produto['DTPROGENTAF'], 0, 10)
            );
            $query = "SELECT NRLANCTONF
                       FROM NFRELACAFSAI
                       WHERE CDFILIAL    = :P_CDFILIAL    AND
                             NRLANCTONF  = :P_NRLANCTONF  AND
                             CDFILDEST   = :P_CDFILDEST   AND
                             CDFILLANC   = :P_CDFILLANC   AND
                             NRAUTOFORN  = :P_NRAUTOFORN  AND
                             CDMARCA     = :P_CDMARCA     AND
                             CDPRODUTO   = :P_CDPRODUTO   AND
                             DTPROGENTAF = :P_DTPROGENTAF";
   
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqNFRELACPEXOL($cdfiliopelog, $nrlanctonf, $cdfildest, $cdfillanc, $produto, $data = null) {
        try {
            $params = array(
                "P_CDFILIAL"    => $cdfiliopelog,
                "P_NRLANCTONF"  => $nrlanctonf,
                "P_CDFILDEST"   => $cdfildest,
                "P_CDFILLANC"   => $cdfillanc,
                "P_NRPEDEXOL"   => $produto['NRAUTOFORN'],
                "P_CDMARCA"     => $produto['CDMARCAOL'],
                "P_CDPRODUTO"   => $produto['CDPRODUTOEST'],
                "P_DTPROGENPEOL"=> $data == null ? substr($produto['DTPROGENPEOL'], 0, 10) : substr($data, 0, 10)
            );
            $query = "SELECT NRLANCTONF
                      FROM NFRELACPEXOL
                      WHERE CDFILIAL     = :P_CDFILIAL    AND
                            NRLANCTONF   = :P_NRLANCTONF  AND
                            CDFILDEST    = :P_CDFILDEST   AND
                            CDFILLANC    = :P_CDFILLANC   AND
                            NRPEDEXOL    = :P_NRPEDEXOL   AND
                            CDMARCA      = :P_CDMARCA     AND
                            CDPRODUTO    = :P_CDPRODUTO   AND
                            DTPROGENPEOL = :P_DTPROGENPEOL";
   
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqExistItem($cdfildest, $cdfillanc, $produto) {
        try {
            $params = array(
                "P_CDFILDEST"   => $cdfildest,
                "P_CDFILLANC"   => $cdfillanc,
                "P_NRPEDEXOL"   => $produto['NRAUTOFORN'],
                "P_CDMARCA"     => $produto['CDMARCAOL'],
                "P_CDPRODUTO"   => $produto['CDPRODUTO'],
                "P_DTPROGEMBARQ"=> substr($produto['DTPROGEMBARQ'], 0, 10)
            );
            $query = "SELECT
                        DTPROGENPEOL
                     FROM
                        ITEMPEDEXOL
                     WHERE
                        CDFILDEST = :P_CDFILDEST AND
                        CDFILLANC = :P_CDFILLANC AND
                        NRPEDEXOL = :P_NRPEDEXOL AND
                        CDMARCA = :P_CDMARCA AND
                        CDPRODUTO = :P_CDPRODUTO AND
                        DTPROGEMBARQ = :P_DTPROGEMBARQ";
   
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmInsNfRelacAfSai($cdfilial, $cdfillanc, $nrlanctonf, $cdfildest, $produto) {
        $sql = "INSERT INTO NFRELACAFSAI
                (CDFILIAL, CDFILLANC, NRLANCTONF, CDFILDEST,
                 NRAUTOFORN, CDPRODUTO, CDMARCA,
                 DTPROGENTAF, QTNFRELAFSAI, VRNFRELAFSAI)
                VALUES
                (:CDFILIAL, :CDFILLANC, :NRLANCTONF, :CDFILDEST,
                 :NRAUTOFORN, :CDPRODUTO, :CDMARCA, :DTPROGENTAF,
                 :QTNFRELAFSAI, :VRNFRELAFSAI)";
                  
        $params = array(
            "CDFILIAL"      => $cdfilial,
            "CDFILLANC"     => $cdfillanc,
            "NRLANCTONF"    => $nrlanctonf,
            "CDFILDEST"     => $cdfildest,
            "NRAUTOFORN"    => $produto['NRAUTOFORN'],
            "CDPRODUTO"     => $produto['CDPRODUTO'],
            "CDMARCA"       => $produto['CDMARCAOL'],
            "DTPROGENTAF"   => substr($produto['DTPROGENTAF'], 0, 10),
            "QTNFRELAFSAI"  => $produto['QTPROGENTAF'],
            "VRNFRELAFSAI"  => $produto['VRPREUNIAF']
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function acmInsNFRELACPEXOL($cdfilial, $cdfillanc, $nrlanctonf, $cdfildest, $produto, $data, $loopItem) {
        $sql = "INSERT INTO NFRELACPEXOL
                 (CDFILIAL, CDFILLANC, NRLANCTONF, CDFILDEST,
                  NRPEDEXOL, CDPRODUTO, CDMARCA,
                  DTPROGENPEOL, QTNFRELPEXOL, VRNFRELPEXOL, NRSEQITPEOL)
                 VALUES
                 (:CDFILIAL, :CDFILLANC, :NRLANCTONF, :CDFILDEST,
                  :NRPEDEXOL, :CDPRODUTO, :CDMARCA, :DTPROGENPEOL,
                  :QTNFRELPEXOL, :VRNFRELPEXOL, LPAD(:NRSEQITPEOL, 4, '0'))";
                  
        $params = array(
            "CDFILIAL"      => $cdfilial,
            "CDFILLANC"     => $cdfillanc,
            "NRLANCTONF"    => $nrlanctonf,
            "CDFILDEST"     => $cdfildest,
            "NRPEDEXOL"     => $produto['NRAUTOFORN'],
            "CDPRODUTO"     => $produto['CDPRODUTO'],
            "CDMARCA"       => $produto['CDMARCAOL'],
            "DTPROGENPEOL"  => $data == null ? substr($produto['DTPROGENTAF'], 0, 10) : substr($data, 0, 10),
            "QTNFRELPEXOL"  => $produto['QTPROGENTAF'],
            "VRNFRELPEXOL"  => $produto['VRPREUNIAF'],
            "NRSEQITPEOL"   => $loopItem
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function acmDelReserva($cdfiliopelog) {
        $sql = "DELETE
                  FROM RESERVAPEDEXOL R
                 WHERE R.CDFILRESOL = :P_FILOPELOG
                   AND EXISTS (SELECT 1
                                 FROM ITEMPEDEXOL I
                                WHERE I.CDFILDEST    = R.CDFILDEST
                                  AND I.CDFILLANC    = R.CDFILLANC
                                  AND I.NRPEDEXOL    = R.NRPEDEXOL
                                  AND I.CDPRODUTO    = R.CDPRODUTO
                                  AND I.CDMARCA      = R.CDMARCA
                                  AND I.DTPROGENPEOL = R.DTPROGENPEOL
                                  AND NVL(I.IDBAIXAOLSAI, 'O') IN ('1', '2', '3'))";
                  
        $params = array(
            "P_FILOPELOG"   => $cdfiliopelog
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function adqDadosAdicAF($cdfiliopelog, $nrlanctonf) {
        try {
            $params = array(
                "P_CDFILIAL"    => $cdfiliopelog,
                "P_NRLANCTONF"  => $nrlanctonf
            );
            $query = "SELECT N.CDFILIEMIT, F.NMFILIAL AS NMFILIEMIT, D.CDFILIAL AS CDFILIRECE, D.NMFILIAL AS NMFILIRECE,
                        F.CDFILIFISC, C.NMFILIAL AS NMFILIFISC,
                        E.DSENDEFILI AS DSENDEFILISEP, E.NRENDEFILISEP, E.DSCOMPENDFIL, E.NMBAIRFILI,
                        M.NMMUNICIPIO, M.SGESTADO, P.NMPAIS, E.NRCEPFILI
                  FROM NOTAFISCAL N, FILIAL F, FILIAL C, FILIAL D, ENDEFILI E, PAIS P, MUNICIPIO M
                 WHERE N.CDFILIAL = :P_CDFILIAL
                   AND N.NRLANCTONF = :P_NRLANCTONF
                   AND F.CDFILIAL = N.CDFILIEMIT
                   AND F.CDFILIFISC = C.CDFILIAL(+)
                   AND D.CDFILIAL = N.CDFILIRECE
                   AND E.CDFILIAL = N.CDFILIRECE
                   AND E.IDTPENDEFILI = 'P'
                   AND P.CDPAIS = E.CDPAIS
                   AND M.CDPAIS = E.CDPAIS
                   AND M.SGESTADO = E.SGESTADO
                   AND M.CDMUNICIPIO = E.CDMUNICIPIO
                   AND NOT EXISTS (SELECT CDFILIAL
                                     FROM ENDEFILI
                                    WHERE ENDEFILI.CDFILIAL = N.CDFILIRECE
                                      AND ENDEFILI.IDTPENDEFILI = 'T')
                UNION
                SELECT N.CDFILIEMIT, F.NMFILIAL AS NMFILIEMIT, D.CDFILIAL AS CDFILIRECE, D.NMFILIAL AS NMFILIRECE,
                        F.CDFILIFISC, C.NMFILIAL AS NMFILIFISC,
                        E.DSENDEFILI AS DSENDEFILISEP, E.NRENDEFILISEP, E.DSCOMPENDFIL, E.NMBAIRFILI,
                        M.NMMUNICIPIO, M.SGESTADO, P.NMPAIS, E.NRCEPFILI
                  FROM NOTAFISCAL N, FILIAL F, FILIAL C, FILIAL D, ENDEFILI E, PAIS P, MUNICIPIO M
                 WHERE N.CDFILIAL = :P_CDFILIAL
                   AND N.NRLANCTONF = :P_NRLANCTONF
                   AND F.CDFILIAL = N.CDFILIEMIT
                   AND F.CDFILIFISC = C.CDFILIAL(+)
                   AND D.CDFILIAL = N.CDFILIRECE
                   AND E.CDFILIAL = N.CDFILIRECE
                   AND E.IDTPENDEFILI = 'T'
                   AND P.CDPAIS = E.CDPAIS
                   AND M.CDPAIS = E.CDPAIS
                   AND M.SGESTADO = E.SGESTADO
                   AND M.CDMUNICIPIO = E.CDMUNICIPIO";
   
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function procedureIncluiReservaOL($row) {
        $procedure = new StoredProcedure($this->connection, 'INCLUI_RESERVA_OL');
        $procedure->addParam(new Param('P_FILOPELOG',       Param::PARAM_INPUT,     $row['P_FILOPELOG'],    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTENTREGA',       Param::PARAM_INPUT,     $row['P_DTENTREGA'],    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DATAENT',         Param::PARAM_INPUT,     $row['P_DATAENT'],      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ROTA',            Param::PARAM_INPUT,     $row['P_ROTA'],         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_PEDOLEXTR',       Param::PARAM_INPUT,     $row['P_PEDOLEXTR'],    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDLOTENVIOL',     Param::PARAM_INPUT,     $row['P_CDLOTENVIOL'],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SUBGRUPO',        Param::PARAM_INPUT,     $row['P_SUBGRUPO'],     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_STATUS',          Param::PARAM_INPUT,     $row['P_STATUS'],       Param::PARAM_TYPE_STR));

        $procedure->addParam(new Param('P_ROWSAFFECT',      Param::PARAM_OUTPUT,    NULL,                   Param::PARAM_TYPE_STR, 10));
        
        $procedure->addParam(new Param('P_VERIFICACNPJ',    Param::PARAM_INPUT,     $row['P_VERIFICACNPJ'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRINSJUREMPR',    Param::PARAM_INPUT,     $row['P_NRSJUREMPR'],   Param::PARAM_TYPE_STR));
        
        return $procedure->execute();
    }
    
    public function getParamSupri($cdfilial) {
        $sql = "SELECT NVL(PARASUPRI.IDUTILCONSSOLIOL, 'N') IDUTILCONSSOLIOL
                FROM FILIAL,
                  PARASUPRI
                WHERE FILIAL.CDFILIAL                         = :P_CDFILDEST
                AND NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) = PARASUPRI.CDFILIAL";
            
        $params = array("P_CDFILDEST" => $cdfilial);
                        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }
    
    public function adqPedidoExol1($cdfiliopelog, $pedido) {
        try {
            $params = array(
                "P_CDFILIALOL"  => $cdfiliopelog,
                "P_USRNRPEDIDO" => $pedido
            );
            $query = "SELECT P.CDFILLANC, P.CDFILDEST, F.NMFILIAL, P.NRPEDEXOL, P.CDFILIOPELOG,
                           CLIENTE.NRINSJURCLIE, CLIENTE.NMFANTCLIE, CLIENTE.IDTPIJURCLIE,
                           NVL(P.USR_IDUTPEDINTEGRAL, 'N') IDUTPEDINTEGRAL,
                           NVL(P.USR_IDAPROVFINANC, 'N') IDAPROVFINANC,
                           NVL(P.USR_IDAPROVALORMIN, 'N') IDAPROVALORMIN,
                           MAX(IT.DTPROGENPEOL) DTPROGENPEOL
                      FROM PEDIDOEXOL P INNER JOIN ITEMPEDEXOL IT ON P.CDFILDEST = IT.CDFILDEST
                                                                 AND P.CDFILLANC = IT.CDFILLANC
                                                                 AND P.NRPEDEXOL = IT.NRPEDEXOL
                                        INNER JOIN FILIAL F ON P.CDFILDEST = F.CDFILIAL
                           LEFT JOIN CLIENTE ON P.CDCLIENTEEXT = CLIENTE.CDCLIENTE
                     WHERE P.USR_NRPEDIDO  = :P_USRNRPEDIDO
                       AND P.CDFILIOPELOG  = :P_CDFILIALOL
                       AND NVL(IT.IDBAIXAOLSAI, '1') IN ('1', '2')
                     GROUP BY P.CDFILLANC, P.CDFILDEST, F.NMFILIAL, P.NRPEDEXOL, P.CDFILIOPELOG,
                          CLIENTE.NRINSJURCLIE, CLIENTE.NMFANTCLIE, CLIENTE.IDTPIJURCLIE,
                          NVL(P.USR_IDUTPEDINTEGRAL, 'N'),
                          NVL(P.USR_IDAPROVFINANC, 'N'),
                          NVL(P.USR_IDAPROVALORMIN, 'N')";
   
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqVariacao($cdproduto) {
        try {
            $params = array(
                "P_CDPRODUTO"  => $cdproduto
            );
            $query = "SELECT PRODUTO.CDPRODUTO PRODUTO_OL, PR.CDPRODUTO PRODUTO, TO_CHAR(NVL(PR.VRARREDCMP, 1)) VARIACAO, RELACPRODOL.CDMARCA
                         FROM RELACPRODOL, PRODUTO, PRODUTO PR
                        WHERE RELACPRODOL.CDPRODARVOL = PRODUTO.CDPRODUTO
                          AND PRODUTO.CDARVPROD = TO_CHAR(:P_CDPRODUTO)
                          AND PR.CDPRODUTO = RELACPRODOL.CDPRARVORIG";
   
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmUpdQuantZero($pedido, $variacao) {
        try {
            $sql = "UPDATE ITEMPEDEXOL
                        SET ITEMPEDEXOL.IDBAIXAOLSAI = '3',
                            ITEMPEDEXOL.IDBAIXENPEOL = '3',
                            ITEMPEDEXOL.QTATENDPEDOL = 0
                      WHERE ITEMPEDEXOL.CDFILDEST    = :P_CDFILIDEST
                        AND ITEMPEDEXOL.CDFILLANC    = :P_CDFILILANC
                        AND ITEMPEDEXOL.NRPEDEXOL    = :P_NRPEDIDO
                        AND ITEMPEDEXOL.DTPROGENPEOL = TO_DATE(:P_DTPROGENPEOL, 'DD/MM/YYYY')
                        AND ITEMPEDEXOL.CDMARCA      = :P_CDMARCA
                        AND ITEMPEDEXOL.CDPRODUTO    = :P_CDPRODUTO";
                  
            $params = array(
                "P_CDFILIDEST" => $pedido['CDFILDEST'],
                "P_CDFILILANC" => $pedido['CDFILLANC'],
                "P_NRPEDIDO" => $pedido['NRPEDEXOL'],
                "P_DTPROGENPEOL" => $pedido['DTPROGENPEOL'],
                "P_CDMARCA" => $variacao['CDMARCA'],
                "P_CDPRODUTO" => $variacao['PRODUTO'],
            );
            
            $query = $this->connection->prepare($sql);
            $query->execute($params);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmDelReservaimporta($pedido, $variacao) {
        try {
            $sql = "DELETE FROM RESERVAPEDEXOL
                      WHERE CDFILDEST     = :CDFILDEST
                        AND CDFILLANC     = :CDFILLANC
                        AND NRPEDEXOL     = :NRPEDEXOL
                        AND DTPROGENPEOL  = :DTPROGENPEOL
                        AND CDMARCA       = :CDMARCA
                        AND CDPRODUTO     = :CDPRODUTO";
                  
            $params = array(
                "CDFILDEST" => $pedido['CDFILDEST'],
                "CDFILLANC" => $pedido['CDFILLANC'],
                "NRPEDEXOL" => $pedido['NRPEDEXOL'],
                "DTPROGENPEOL" => $pedido['DTPROGENPEOL'],
                "CDMARCA" => $variacao['CDMARCA'],
                "CDPRODUTO" => $variacao['PRODUTO'],
            );
            
            $query = $this->connection->prepare($sql);
            $query->execute($params);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqQuantAlterada($pedido, $variacao) {
        try {
            $params = array(
                "P_CDFILIDEST" => $pedido['CDFILDEST'],
                "P_CDFILILANC" => $pedido['CDFILLANC'],
                "P_NRPEDIDO" => $pedido['NRPEDEXOL'],
                "P_DTPROGENPEOL" => $pedido['DTPROGENPEOL'],
                "P_CDMARCA" => $variacao['CDMARCA'],
                "P_CDPRODUTO" => $variacao['PRODUTO'],
            );
            $query = "SELECT ITEMPEDEXOL.NRPEDEXOL
                        FROM ITEMPEDEXOL
                       WHERE ITEMPEDEXOL.CDFILDEST     = :P_CDFILIDEST
                         AND ITEMPEDEXOL.CDFILLANC     = :P_CDFILILANC
                         AND ITEMPEDEXOL.NRPEDEXOL     = :P_NRPEDIDO
                         AND ITEMPEDEXOL.DTPROGENPEOL  = TO_CHAR(:P_DTPROGENPEOL)
                         AND ITEMPEDEXOL.CDPRODUTO = TO_CHAR(:P_CDPRODUTO)
                         AND ITEMPEDEXOL.CDMARCA = :P_CDMARCA";
   
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmUpdItPexol($pedido, $variacao, $item, $var) {
        try {
            $sql = "UPDATE ITEMPEDEXOL
                     SET ITEMPEDEXOL.QTATENDPEDOL  = DECODE (:P_VARIACAO, 'CX', :P_QTPROGENPEOL, :PESO_LIQ),
                         ITEMPEDEXOL.QTVOLUPEDOL   = :P_QTVOLUPEDOL
                   WHERE ITEMPEDEXOL.CDFILDEST     = :P_CDFILIDEST
                     AND ITEMPEDEXOL.CDFILLANC     = :P_CDFILILANC
                     AND ITEMPEDEXOL.NRPEDEXOL     = :P_NRPEDIDO
                     AND ITEMPEDEXOL.DTPROGENPEOL  = TO_DATE(:P_DTPROGENPEOL, 'DD/MM/YYYY')
                     AND ITEMPEDEXOL.CDMARCA = :P_CDMARCA
                     AND ITEMPEDEXOL.CDPRODUTO = TO_CHAR(:P_CDPRODUTO)";
                  
            $params = array(
                "P_CDFILIDEST" => $pedido['CDFILDEST'],
                "P_CDFILILANC" => $pedido['CDFILLANC'],
                "P_NRPEDIDO" => $pedido['NRPEDEXOL'],
                "P_DTPROGENPEOL" => substr($pedido['DTPROGENPEOL'], 0, 10),
                "P_CDMARCA" => $variacao['CDMARCA'],
                "P_CDPRODUTO" => $variacao['PRODUTO'],
                "P_QTPROGENPEOL" => $item['QTDE_PRODUTO'],
                "P_QTVOLUPEDOL" => $item['QTDE_VOLUMES'],
                "P_VARIACAO" => $var,
                "PESO_LIQ" => $item['PESO_LIQ_ITEM']
            );
            
            $query = $this->connection->prepare($sql);
            $query->execute($params);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmUpdReserva($pedido, $variacao, $item) {
        try {
            $sql = "UPDATE RESERVAPEDEXOL
                      SET QTRESERVESTOL = :P_QTPROGENPEOL
                    WHERE CDFILDEST     = :P_CDFILIDEST
                      AND CDFILLANC     = :P_CDFILILANC
                      AND NRPEDEXOL     = :P_NRPEDIDO
                      AND DTPROGENPEOL  = :P_DTPROGENPEOL
                      AND CDMARCA       = :P_CDMARCA
                      AND CDPRODUTO     = :P_CDPRODUTO";
                  
            $params = array(
                "P_CDFILIDEST" => $pedido['CDFILDEST'],
                "P_CDFILILANC" => $pedido['CDFILLANC'],
                "P_NRPEDIDO" => $pedido['NRPEDEXOL'],
                "P_DTPROGENPEOL" => $pedido['DTPROGENPEOL'],
                "P_CDMARCA" => $variacao['CDMARCA'],
                "P_CDPRODUTO" => $variacao['PRODUTO'],
                "P_QTPROGENPEOL" => $item['QTDE_PRODUTO']
            );
            
            $query = $this->connection->prepare($sql);
            $query->execute($params);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqItemAutF($item) {
        try {
            $params = array(
                "P_NRAUTOFORN" => $item['NUM_PEDIDO'],
                "P_CDPRODUTO"  => $item['COD_PRODUTO']
            );
            $query = "SELECT NRAUTOFORN, CDFILDEST, CDFILLANC, I.CDPRODUTO, I.CDMARCA, QTPROGENTAF, I.DTPROGENTAF
                       FROM ITEMAUTF I, RELACPRODOL R
                       WHERE NRAUTOFORN    = :P_NRAUTOFORN
                       AND R.CDPRODARVOL = :P_CDPRODUTO
                       AND CDPRODUTO     = R.CDPRARVORIG
                       AND I.CDMARCA     = R.CDMARCA
                       AND NVL(I.IDITAFMIGRADA, 'N') = 'N'";
   
            $std = $this->connection->prepare($query);
            $std->execute($params);
                
            $resultado = $std->fetchAll();
                
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmUpdItemAutF($itemAF, $item) {
        try {
            $sql = "UPDATE ITEMAUTF
                        SET QTATENDPEDOL  = :P_QTATENDPEDOL,
                            IDTPGERARUP   = NULL,
                            IDITMSEPARADO = DECODE(:P_QTATENDPEDOL, 0, 'N', 'S')
                      WHERE CDFILDEST     = :P_CDFILIDEST
                        AND CDFILLANC     = :P_CDFILLANC
                        AND NRAUTOFORN    = :P_NRAUTOFORN
                        AND DTPROGENTAF   = TO_DATE(:DTPROGENTAF, 'DD/MM/YYYY')
                        AND CDPRODUTO     = TO_CHAR(:P_CDPRODUTO)
                        AND CDMARCA       = :P_CDMARCA
                        AND IDBAIXITMAF IS NULL";
                  
            $params = array(
                "P_CDFILIDEST" => $itemAF['CDFILDEST'],
                "P_CDFILILANC" => $itemAF['CDFILLANC'],
                "P_NRAUTOFORN" => $item['NUM_PEDIDO'],
                "P_CDPRODUTO" => $itemAF['CDPRODUTO'],
                "DTPROGENTAF" => $itemAF['DTPROGENPEOL'],
                "P_CDMARCA" => $itemAF['CDMARCA'],
                "P_QTATENDPEDOL" => $item['QTDE_PRODUTO'],
                "P_ATUALIZAQTAF" => $item['QTDE_PRODUTO']
            );
            
            $query = $this->connection->prepare($sql);
            $query->execute($params);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmUpdItemAutF1($itemAF, $item) {
        try {
            $sql = "UPDATE ITEMAUTF
                        SET QTATENDPEDOL  = :P_QTATENDPEDOL,
                            IDTPGERARUP   = 'U',
                            IDITMSEPARADO = 'N'
                      WHERE CDFILDEST     = :P_CDFILIDEST
                        AND CDFILLANC     = :P_CDFILLANC
                        AND NRAUTOFORN    = :P_NRAUTOFORN
                        AND DTPROGENTAF   = TO_DATE(:DTPROGENTAF, 'DD/MM/YYYY')
                        AND CDPRODUTO     = TO_CHAR(:P_CDPRODUTO)
                        AND CDMARCA       = :P_CDMARCA
                        AND IDBAIXITMAF IS NULL";
                  
            $params = array(
                "P_CDFILIDEST" => $itemAF['CDFILDEST'],
                "P_CDFILILANC" => $itemAF['CDFILLANC'],
                "P_NRAUTOFORN" => $item['NUM_PEDIDO'],
                "P_CDPRODUTO" => $itemAF['CDPRODUTO'],
                "DTPROGENTAF" => $itemAF['DTPROGENPEOL'],
                "P_CDMARCA" => $itemAF['CDMARCA'],
                "P_QTATENDPEDOL" => $item['QTDE_PRODUTO']
            );
            
            $query = $this->connection->prepare($sql);
            $query->execute($params);
        } catch(Exception $e) {
            throw $e;
        }
    }
}
