<?php
namespace Teknisa\OPL\Services;

use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class Opl01600ParametrizacaoDoOPLService {
  protected $connection;
  protected $newCode;
    
  public function __construct($entityManager, $environment, $newCode) {
    $this->entityManager  = $entityManager;
    $this->connection     = $entityManager->getConnection();
    $this->environment    = $environment;
    $this->newCode        = $newCode;
  }
    
  public function realizaCommit() {
    $this->entityManager->commit();
  }
    
  public function realizaRollback() {
    $this->entityManager->rollBack();
  }
    
  public function beginTransaction() {
    $this->entityManager->beginTransaction();
  }
  
  public function inclusaoAutTipoOperOPL($row) {
    try {
      $procedure = new StoredProcedure($this->connection, 'CAD_AUT_TIPOOPEROPL');
      $procedure->addParam(new Param('P_CDFILIOPELOG', Param::PARAM_INPUT, $row['CDFILIOPELOG'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDFILDEST', Param::PARAM_INPUT, $row['CDFILDEST'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDTIPOOPERSAID', Param::PARAM_INPUT, $row['CDTIPOOPERSAID'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDTIPOOPERENTR', Param::PARAM_INPUT, $row['CDTIPOOPERENTR'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_OPERACAO', Param::PARAM_INPUT, 'I', Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $this->environment->getCdOperador(), Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_ERROS', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 1));
      $procedure->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, $row['CDCLIENTE'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_VRFATODESONERA', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDSERIE', Param::PARAM_INPUT, $row['CDSERIE'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDTPOPSACAIXA', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDTPOPENCAIXA', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
      
      return $procedure->execute();
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function exclusaoAutTipoOperOPL($row) {
    try {
      $procedure = new StoredProcedure($this->connection, 'CAD_AUT_TIPOOPEROPL');
      $procedure->addParam(new Param('P_CDFILIOPELOG', Param::PARAM_INPUT, $row['CDFILIOPELOG'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDFILDEST', Param::PARAM_INPUT, $row['CDFILDEST'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDTIPOOPERSAID', Param::PARAM_INPUT, 'T', Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDTIPOOPERENTR', Param::PARAM_INPUT, 'T', Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_OPERACAO', Param::PARAM_INPUT, 'E', Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $this->environment->getCdOperador(), Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_ERROS', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 1));
      $procedure->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_VRFATODESONERA', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDSERIE', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDTPOPSACAIXA', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDTPOPENCAIXA', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
      
      return $procedure->execute();
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function acrAutParamProdSupFili($row) {
    try {
      $procedure = new StoredProcedure($this->connection, 'CAD_AUT_PARAMPRODSUP_FILI');
      $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row['CDFILORIG'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDFILDEST', Param::PARAM_INPUT, $row['CDFILDEST'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_SUBSTITUI', Param::PARAM_INPUT, 'N', Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_ERROS', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 1));
      
      return $procedure->execute();
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function subAutParamProdSupFili($row) {
    try {
      $procedure = new StoredProcedure($this->connection, 'CAD_AUT_PARAMPRODSUP_FILI');
      $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row['CDFILORIG'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDFILDEST', Param::PARAM_INPUT, $row['CDFILDEST'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_SUBSTITUI', Param::PARAM_INPUT, 'S', Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_ERROS', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 1));
      
      return $procedure->execute();
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function inclusaoCadAutParamProdSup($row) {
    try {
      $procedure = new StoredProcedure($this->connection, 'CAD_AUT_PARAMPRODSUP');
      $procedure->addParam(new Param('P_CDFILIOPELOG', Param::PARAM_INPUT, $row['CDFILIOPELOG'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDREGIAO', Param::PARAM_INPUT, $row['CDREGIAO'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDFILDEST', Param::PARAM_INPUT, $row['CDFILIAL'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDGRUPPROD', Param::PARAM_INPUT, $row['CDGRUPPROD'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDSUBGRPROD', Param::PARAM_INPUT, $row['CDSUBGRPROD'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDFILISUBGR', Param::PARAM_INPUT, 'P', Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_OPERACAO', Param::PARAM_INPUT, 'I', Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDPRODINI', Param::PARAM_INPUT, $row['CDPRODINI'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDPRODFIN', Param::PARAM_INPUT, $row['CDPRODFIN'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_ERROS', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 1));
      
      return $procedure->execute();
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function exclusaoCadAutParamProdSup($row) {
    try {
      $procedure = new StoredProcedure($this->connection, 'CAD_AUT_PARAMPRODSUP');
      $procedure->addParam(new Param('P_CDFILIOPELOG', Param::PARAM_INPUT, $row['CDFILIOPELOG'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDREGIAO', Param::PARAM_INPUT, $row['CDREGIAO'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDFILDEST', Param::PARAM_INPUT, $row['CDFILIAL'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDGRUPPROD', Param::PARAM_INPUT, $row['CDGRUPPROD'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDSUBGRPROD', Param::PARAM_INPUT, $row['CDSUBGRPROD'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDFILISUBGR', Param::PARAM_INPUT, 'P', Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_OPERACAO', Param::PARAM_INPUT, 'E', Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDPRODINI', Param::PARAM_INPUT, $row['CDPRODINI'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDPRODFIN', Param::PARAM_INPUT, $row['CDPRODFIN'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_ERROS', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 1));
      
      return $procedure->execute();
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function inclusaoCadAutRelacProdOL($row) {
    try {
      $procedure = new StoredProcedure($this->connection, 'CAD_AUT_RELACPRODOL');
      $procedure->addParam(new Param('P_CDPRODINIOL', Param::PARAM_INPUT, $row['CDPRODINIOL'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDPRODFINOL', Param::PARAM_INPUT, $row['CDPRODFINOL'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_OPERACAO', Param::PARAM_INPUT, 'I', Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_ERROS', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 1));
      $procedure->addParam(new Param('P_CDPRODINI', Param::PARAM_INPUT, $row['CDPRODINI'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDPRODFIN', Param::PARAM_INPUT, $row['CDPRODFIN'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDMARCA', Param::PARAM_INPUT, $row['CDMARCA'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDFILIOPELOG', Param::PARAM_INPUT, $row['CDFILIOPELOG'], Param::PARAM_TYPE_STR));
      
      return $procedure->execute();
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function exclusaoCadAutRelacProdOL($row) {
    try {
      $procedure = new StoredProcedure($this->connection, 'CAD_AUT_RELACPRODOL');
      $procedure->addParam(new Param('P_CDPRODINIOL', Param::PARAM_INPUT, $row['CDPRODINIOL'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDPRODFINOL', Param::PARAM_INPUT, $row['CDPRODFINOL'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_OPERACAO', Param::PARAM_INPUT, 'E', Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_ERROS', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 1));
      $procedure->addParam(new Param('P_CDPRODINI', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDPRODFIN', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDMARCA', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDFILIOPELOG', Param::PARAM_INPUT, 'P', Param::PARAM_TYPE_STR));
      return $procedure->execute();
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function getGridParametrizacaoDeProdutoPorUnidade($row) {
    try {
      $params = array(
        "P_NMPRODINI"         => $row['P_NMPRODINI'],
        "P_NMPRODFIN"         => $row['P_NMPRODFIN'],
        "P_CDFILIAL"          => $row['P_CDFILIAL'],
        "P_CDPRODINI"         => $row['P_CDPRODINI'],
        "P_CDPRODFIN"         => $row['P_CDPRODFIN'],
        "P_CDFILIOL"          => $row['P_CDFILIOL'],
        "P_TCDFILIOL"         => $row['P_TCDFILIOL'],
        "P_TCDFILIAL"         => $row['P_TCDFILIAL'],
        "P_CDPRODFINTEXT"     => $row['P_CDPRODFINTEXT'],
        "P_IDUTLSUBGRPRODFIL" => $row['P_IDUTLSUBGRPRODFIL'],
        "P_CDFILIOPELOG"      => $row['P_CDFILIOPELOG'],
        "P_IDITEMSEMPARAM"    => $row['P_IDITEMSEMPARAM'],
        "P_CDGRUPPROD"        => $row['P_CDGRUPPROD'],
        "P_CDSUBGRARVOR"      => $row['P_CDSUBGRARVOR'],
        "P_SUBGRUPO"          => $row['P_SUBGRUPO']
      );
      $query = "SELECT NRSEQPARPROD, CDFILIOPELOG, NMFILIALOL, CDFILIAL, NMFILIAL, CDPRODUTO, CDARVPROD, NMSUBGRPROD, NMPRODUTO, SGUNIDADE, CDGRUPPROD, CDSUBGRPROD, CDFILISUBGR, :P_CDPRODINI CDPRODINI, :P_CDPRODFINTEXT CDPRODFIN, :P_NMPRODINI NMPRODINI, :P_NMPRODFIN NMPRODFIN, NRORG FROM TABLE ( QRY_PROD_FILIAL_OL(:P_CDFILIAL, :P_TCDFILIAL, :P_CDFILIOL, :P_TCDFILIOL, :P_CDFILIOPELOG, :P_CDPRODINI, :P_CDPRODFIN, :P_SUBGRUPO, :P_CDGRUPPROD, :P_CDSUBGRARVOR, :P_IDITEMSEMPARAM, :P_IDUTLSUBGRPRODFIL) ) ORDER BY CDFILIOPELOG, CDFILIAL, CDSUBGRPROD, CDARVPROD";
      
      $std = $this->connection->prepare($query);
      $std->execute($params);
            
      $resultado = $std->fetchAll();
            
      return $resultado;
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function updateEmbalagemAtivaProd($row) {
    try {
      $params = array(
        "P_IDPRCMPATIVO"  => $row['NEWIDPRCMPATIVO'],
        "P_CDPRARVORIG"   => $row['CDPRODUTO']
      );
      
      $query = "UPDATE PRODUTO SET IDPRCMPATIVO = :P_IDPRCMPATIVO WHERE CDPRODUTO = :P_CDPRARVORIG";
      
      $std = $this->connection->prepare($query);
      $std->execute($params);
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function updateConversaoUnEstProd($row) {
    try {
      $params = array(
        "P_VRFATUNEST"  => $row['VRFATUNEST'],
        "P_CDPRODARVOL" => $row['CDPRODUTOL']
      );
      
      $query = "UPDATE CNVPRODUNEST SET VRFATUNEST = :P_VRFATUNEST WHERE CDPRUNEST = :P_CDPRODARVOL";
      
      $std = $this->connection->prepare($query);
      $std->execute($params);
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function updateProdutoOL($row) {
    try {
      $params = array(
        "P_CDPRODARVOL_NOVO"    => $row['NEWCDPRODUTOL'],
        "P_CDPRARVORIG"         => $row['CDPRODUTO'],
        "P_CDMARCA"             => $row['CDMARCA'],
        "P_CDPRODARVOL_ANTIGO"  => $row['CDPRODUTOL']
      );
      $query = "UPDATE RELACPRODOL SET CDPRODARVOL = :P_CDPRODARVOL_NOVO WHERE CDPRARVORIG = :P_CDPRARVORIG AND CDMARCA = :P_CDMARCA AND CDPRODARVOL = :P_CDPRODARVOL_ANTIGO";
      
      $std = $this->connection->prepare($query);
      $std->execute($params);
    } catch(Exception $e) {
      throw $e;
    }
  }
}