/*
 Tecnisa - Tecnologia em Informtica

 Nome: Atualiza_Saldo_Bancario

 Descrio:
 Procedimento para evoluo/atualizao dos saldos da conta corrente.

 Criao:
 24/03/2005 - Jenilson Ferreira

 Chamada para criao
 @C:\Win_V4\Sql\End_Conv\Oracle\1016_FINSTPO_ATUALIZA_SALDO_BANCARIO.SQL

 */

CREATE OR REPLACE PROCEDURE ATUALIZA_SALDO_BANCARIO
   (P_CDREDUCONT  IN SDOCCORR.CDREDUCONT%TYPE,
    P_DTSALDO     IN DATE,
    P_VRSDOCCORR  IN SDOCCORR.VRSDOCCORR%TYPE,
    P_IDATUALIZA  IN VARCHAR2)
AS
   V_LOOP        INTEGER;
   V_DTMOVIMENTO DATE;
   V_VRMOVIMENTO NUMBER;
   V_VRSALDOIMPL NUMBER;
   V_SALDOANT    NUMBER;
BEGIN

   IF P_IDATUALIZA = '0' THEN /* REALIZA A EVOLUO DO SALDO DA CONTA CORRENTE*/
      V_LOOP        := 0;
      V_VRMOVIMENTO := 0;
      V_VRSALDOIMPL := 0;
      V_DTMOVIMENTO := NULL;

      SELECT MAX(DTSDOCCORR)
      INTO V_DTMOVIMENTO
      FROM
         SDOCCORR
      WHERE
         CDREDUCONT = P_CDREDUCONT  AND
         DTSDOCCORR = (SELECT MAX(DTSDOCCORR)
                       FROM SDOCCORR
                       WHERE CDREDUCONT =  P_CDREDUCONT AND
                             DTSDOCCORR <= P_DTSALDO    AND
                             NOT VRSDOIMPLA IS NULL);

      IF V_DTMOVIMENTO IS NULL THEN
         SELECT MAX(DTSDOCCORR)
         INTO V_DTMOVIMENTO
         FROM
            SDOCCORR
         WHERE
            CDREDUCONT = P_CDREDUCONT  AND
            DTSDOCCORR = (SELECT MAX(DTSDOCCORR)
                          FROM SDOCCORR
                          WHERE CDREDUCONT =  P_CDREDUCONT AND
                                DTSDOCCORR <= P_DTSALDO);
      END IF;

      IF V_DTMOVIMENTO IS NULL THEN
         SELECT MIN(DTSDOCCORR)
         INTO V_DTMOVIMENTO
         FROM
            SDOCCORR
         WHERE
            CDREDUCONT = P_CDREDUCONT;
      END IF;
      IF V_DTMOVIMENTO IS NULL  THEN
         V_DTMOVIMENTO := P_DTSALDO;
      END IF;
      FOR C_SALDO IN (SELECT DTMOVBANC,
                             SUM(VRMOVBANC)  AS VRSDOMOV,
                             SUM(IDEXISTS)   AS IDEXISTS,
                             SUM(VRSDOIMPLA) AS VRSDOIMPLA
                      FROM
                         (SELECT
                             DTMOVBANC,
                             VRMOVBANC,
                             0           AS IDEXISTS,
                             0           AS VRSDOIMPLA
                          FROM
                             MOVBANCARIO
                          WHERE
                             (CDREDUCONT = P_CDREDUCONT)  AND
                             (DTMOVBANC >= V_DTMOVIMENTO) AND
                             (IDMOVBANC  = 'C')
                          UNION ALL
                          SELECT
                             DTMOVBANC,
                             VRMOVBANC * -1,
                             0           AS IDEXISTS,
                             0           AS VRSDOIMPLA
                          FROM
                             MOVBANCARIO
                          WHERE
                             (CDREDUCONT = P_CDREDUCONT)  AND
                             (DTMOVBANC >= V_DTMOVIMENTO) AND
                             (IDMOVBANC  = 'D')
                          UNION ALL
                          SELECT
                             DTSDOCCORR  AS DTMOVBANC,
                             0           AS VRMOVBANC,
                             1           AS IDEXISTS,
                             VRSDOIMPLA
                          FROM
                             SDOCCORR
                          WHERE
                             (CDREDUCONT =  P_CDREDUCONT)  AND
                             (DTSDOCCORR >= V_DTMOVIMENTO) AND
                             (NOT VRSDOIMPLA IS NULL)
                          )
                      GROUP BY
                         DTMOVBANC
                      ORDER BY
                         DTMOVBANC) LOOP

         IF V_LOOP = 0 THEN
            DELETE FROM SDOCCORR
            WHERE
               (CDREDUCONT =  P_CDREDUCONT) AND
               (DTSDOCCORR >= V_DTMOVIMENTO);
            V_LOOP := 1;
         END IF;
            
         /* REALIZA A EVOLUCAO DO SALDO DA CONTA CORRENTE  */
         IF C_SALDO.IDEXISTS > 0 THEN
            V_VRMOVIMENTO := ROUND(C_SALDO.VRSDOIMPLA, 2);
            V_VRSALDOIMPL := V_VRMOVIMENTO;
         ELSE
            V_VRMOVIMENTO := ROUND(V_VRMOVIMENTO + C_SALDO.VRSDOMOV, 2);
            V_VRSALDOIMPL := NULL;
         END IF;

         INSERT INTO SDOCCORR
            (CDREDUCONT,
             DTSDOCCORR,
             VRSDOCCORR,
             VRSDOIMPLA)
         VALUES
            (P_CDREDUCONT,
             C_SALDO.DTMOVBANC,
             V_VRMOVIMENTO,
             V_VRSALDOIMPL);
      END LOOP;
   END IF;

   IF P_IDATUALIZA = '1' THEN /* REALIZA A ATUALIZAO DO SALDO DA CONTA CORRENTE*/
      V_LOOP        := 0;
      V_VRMOVIMENTO := ROUND(P_VRSDOCCORR, 2);
      V_SALDOANT    := 0;

      /* RETORNA O SALDO DO DIA ANTERIOR */
      SELECT NVL(SUM(VRSDOCCORR), 0)
      INTO V_SALDOANT
      FROM
         SDOCCORR
      WHERE
         CDREDUCONT = P_CDREDUCONT  AND
         DTSDOCCORR = (SELECT MAX(DTSDOCCORR)
                       FROM SDOCCORR
                       WHERE CDREDUCONT =  P_CDREDUCONT AND
                             DTSDOCCORR <= P_DTSALDO -1);

      FOR C_SALDO IN (SELECT
                         DTSDOCCORR,
                         VRSDOCCORR,
                         VRSDOIMPLA
                      FROM
                         SDOCCORR
                      WHERE
                         (CDREDUCONT =  P_CDREDUCONT) AND
                         (DTSDOCCORR >= P_DTSALDO)
                      ORDER BY
                         DTSDOCCORR) LOOP

          IF (P_DTSALDO < C_SALDO.DTSDOCCORR) AND (V_LOOP = 0) THEN
             /* INSERE O NOVO SALDO */
             INSERT INTO SDOCCORR
                (CDREDUCONT,
                 DTSDOCCORR,
                 VRSDOCCORR,
                 VRSDOIMPLA)
             VALUES
                (P_CDREDUCONT,
                 P_DTSALDO,
                 ROUND(V_SALDOANT + V_VRMOVIMENTO, 2),
                 NULL);
          END IF;
          V_LOOP := 1;

          IF NOT C_SALDO.VRSDOIMPLA IS NULL THEN
             EXIT;
          END IF;

          /* ALTERA O SALDO ATUAL  */
          UPDATE SDOCCORR
          SET VRSDOCCORR = ROUND(VRSDOCCORR + V_VRMOVIMENTO, 2)
          WHERE
             CDREDUCONT = P_CDREDUCONT  AND
             DTSDOCCORR = C_SALDO.DTSDOCCORR;
      END LOOP;

      IF (V_LOOP = 0) THEN
         /* INSERE O NOVO SALDO */
         INSERT INTO SDOCCORR
            (CDREDUCONT,
             DTSDOCCORR,
             VRSDOCCORR,
             VRSDOIMPLA)
         VALUES
            (P_CDREDUCONT,
             P_DTSALDO,
             ROUND(V_SALDOANT + V_VRMOVIMENTO, 2),
             NULL);
      END IF;
   END IF;
END;
/


-- VERSAO --
BEGIN ATUVEROBJ('ATUALIZA_SALDO_BANCARIO','6.49.063'); END;
/
